@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -15,10 +15,11 @@
 
 module Graphics.Rendering.OpenGL.GL (
    -- * OpenGL Operation
-   module Graphics.Rendering.OpenGL.GL.BasicTypes,
+   module Graphics.Rendering.OpenGL.Raw.Core31,
    module Graphics.Rendering.OpenGL.GL.BeginEnd,
    module Graphics.Rendering.OpenGL.GL.VertexSpec,
    module Graphics.Rendering.OpenGL.GL.VertexArrays,
+   module Graphics.Rendering.OpenGL.GL.VertexArrayObjects,
    module Graphics.Rendering.OpenGL.GL.BufferObjects,
    module Graphics.Rendering.OpenGL.GL.Rectangles,
    module Graphics.Rendering.OpenGL.GL.CoordTrans,
@@ -29,6 +30,7 @@ module Graphics.Rendering.OpenGL.GL (
 
    -- * Rasterization
    module Graphics.Rendering.OpenGL.GL.Antialiasing,
+   module Graphics.Rendering.OpenGL.GL.FramebufferObjects,
    module Graphics.Rendering.OpenGL.GL.Points,
    module Graphics.Rendering.OpenGL.GL.LineSegments,
    module Graphics.Rendering.OpenGL.GL.Polygons,
@@ -50,17 +52,26 @@ module Graphics.Rendering.OpenGL.GL (
    module Graphics.Rendering.OpenGL.GL.DisplayLists,
    module Graphics.Rendering.OpenGL.GL.FlushFinish,
    module Graphics.Rendering.OpenGL.GL.Hints,
+   module Graphics.Rendering.OpenGL.GL.PixellikeObject,
+   module Graphics.Rendering.OpenGL.GL.TransformFeedback,
+-- is exported through PerFragment for backwards compatibility reasons
+--   module Graphics.Rendering.OpenGL.GL.QueryObjects,
 
    -- * State and State Requests
-   module Graphics.Rendering.OpenGL.GL.StateVar,
+   module Data.StateVar,
+   module Data.Tensor,
    module Graphics.Rendering.OpenGL.GL.StringQueries,
    module Graphics.Rendering.OpenGL.GL.SavingState
 ) where
 
-import Graphics.Rendering.OpenGL.GL.BasicTypes
+import Graphics.Rendering.OpenGL.Raw.Core31 (
+   GLbitfield,  GLboolean,  GLbyte,  GLchar,  GLclampd,  GLclampf,  GLdouble,
+   GLenum,  GLfloat,  GLhalf,  GLint,  GLintptr,  GLshort,  GLsizei,
+   GLsizeiptr,  GLubyte,  GLuint,  GLushort )
 import Graphics.Rendering.OpenGL.GL.BeginEnd
 import Graphics.Rendering.OpenGL.GL.VertexSpec
 import Graphics.Rendering.OpenGL.GL.VertexArrays
+import Graphics.Rendering.OpenGL.GL.VertexArrayObjects
 import Graphics.Rendering.OpenGL.GL.BufferObjects
 import Graphics.Rendering.OpenGL.GL.Rectangles
 import Graphics.Rendering.OpenGL.GL.CoordTrans
@@ -70,6 +81,7 @@ import Graphics.Rendering.OpenGL.GL.Colors
 import Graphics.Rendering.OpenGL.GL.Shaders
 
 import Graphics.Rendering.OpenGL.GL.Antialiasing
+import Graphics.Rendering.OpenGL.GL.FramebufferObjects
 import Graphics.Rendering.OpenGL.GL.Points
 import Graphics.Rendering.OpenGL.GL.LineSegments
 import Graphics.Rendering.OpenGL.GL.Polygons
@@ -89,7 +101,12 @@ import Graphics.Rendering.OpenGL.GL.Feedback
 import Graphics.Rendering.OpenGL.GL.DisplayLists
 import Graphics.Rendering.OpenGL.GL.FlushFinish
 import Graphics.Rendering.OpenGL.GL.Hints
+import Graphics.Rendering.OpenGL.GL.PixellikeObject
+import Graphics.Rendering.OpenGL.GL.TransformFeedback
+-- is exported through PerFragment for backwards compatibility reasons
+--import Graphics.Rendering.OpenGL.GL.QueryObjects
 
-import Graphics.Rendering.OpenGL.GL.StateVar
+import Data.StateVar
+import Data.Tensor
 import Graphics.Rendering.OpenGL.GL.StringQueries
 import Graphics.Rendering.OpenGL.GL.SavingState
