@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.BufferObjects
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -15,7 +15,7 @@
 
 module Graphics.Rendering.OpenGL.GL.BufferObjects (
    -- * Object Names
-   ObjectName(..),
+   module Data.ObjectName,
 
    -- * Buffer Objects
    BufferObject(BufferObject),
@@ -30,50 +30,38 @@ module Graphics.Rendering.OpenGL.GL.BufferObjects (
    -- * Mapping Buffer Objects
    BufferAccess(..), MappingFailure(..), withMappedBuffer,
    mapBuffer, unmapBuffer,
-   bufferAccess, bufferMapped
+   bufferAccess, bufferMapped,
+
+   BufferRangeAccessBit(..), Offset, Length,
+   mapBufferRange, flushMappedBufferRange,
+
+   -- * Indexed Buffer manipulation
+   BufferIndex,
+   RangeStartIndex, RangeSize,
+   BufferRange,
+   IndexedBufferTarget(..),
+   bindBufferBase, bindBufferRange,
+   indexedBufferStart, indexedBufferSize
 ) where
 
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Marshal.Array ( withArrayLen, peekArray, allocaArray )
-import Foreign.Ptr ( Ptr )
-import Foreign.Storable ( Storable(peek) )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLboolean, GLenum, GLint, GLintptr, GLuint, GLsizei, GLsizeiptr )
-import Graphics.Rendering.OpenGL.GL.Exception ( finallyRet )
-import Graphics.Rendering.OpenGL.GL.Extensions (
-   FunPtr, unsafePerformIO, Invoker, getProcAddress )
-import Graphics.Rendering.OpenGL.GL.GLboolean ( unmarshalGLboolean )
-import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek1 )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetArrayBufferBinding,GetElementArrayBufferBinding,
-            GetVertexArrayBufferBinding,GetNormalArrayBufferBinding,
-            GetColorArrayBufferBinding,GetIndexArrayBufferBinding,
-            GetTextureCoordArrayBufferBinding,GetEdgeFlagArrayBufferBinding,
-            GetFogCoordArrayBufferBinding,GetSecondaryColorArrayBufferBinding,
-            GetPixelPackBufferBinding,GetPixelUnpackBufferBinding,
-            GetCopyReadBuffer,GetCopyWriteBuffer),
-   getInteger1, maybeNullPtr,
-   GetVertexAttribPName(GetVertexAttribArrayBufferBinding), getVertexAttribInteger1 )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar, StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.VertexArrays ( ClientArrayType(..) )
-import Graphics.Rendering.OpenGL.GL.VertexSpec ( AttribLocation )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal ( recordInvalidEnum )
+import Data.ObjectName
+import Data.List(foldl1')
+import Data.Bits((.|.))
+import Data.Maybe
 
---------------------------------------------------------------------------------
+import Data.StateVar
+import Foreign.Marshal.Alloc
+import Foreign.Marshal.Array
+import Foreign.Ptr
+import Foreign.Storable
 
-#include "HsOpenGLExt.h"
-
---------------------------------------------------------------------------------
-
--- | An 'ObjectName' corresponds to the general OpenGL notion of an explicitly
--- handled object name, e.g. a display list name, a texture object name, a
--- buffer object name, etc.
-
-class ObjectName a where
-   genObjectNames :: Int -> IO [a]
-   deleteObjectNames:: [a] -> IO ()
-   isObjectName :: a -> IO Bool
+import Graphics.Rendering.OpenGL.GL.Exception
+import Graphics.Rendering.OpenGL.GL.GLboolean
+import Graphics.Rendering.OpenGL.GL.PeekPoke
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.VertexArrays
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -85,20 +73,14 @@ newtype BufferObject = BufferObject { bufferID :: GLuint }
 instance ObjectName BufferObject where
    genObjectNames n =
       allocaArray n $ \buf -> do
-        glGenBuffersARB (fromIntegral n) buf
+        glGenBuffers (fromIntegral n) buf
         fmap (map BufferObject) $ peekArray n buf
 
    deleteObjectNames bufferObjects =
       withArrayLen (map bufferID bufferObjects) $
-         glDeleteBuffersARB . fromIntegral
+         glDeleteBuffers . fromIntegral
 
-   isObjectName = fmap unmarshalGLboolean . glIsBufferARB . bufferID
-
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glGenBuffersARB,GLsizei -> Ptr GLuint -> IO ())
-
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glDeleteBuffersARB,GLsizei -> Ptr GLuint -> IO ())
-
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glIsBufferARB,GLuint -> IO GLboolean)
+   isObjectName = fmap unmarshalGLboolean . glIsBuffer . bufferID
 
 --------------------------------------------------------------------------------
 
@@ -109,18 +91,20 @@ data BufferTarget =
    | ElementArrayBuffer
    | PixelPackBuffer
    | PixelUnpackBuffer
+   | TransformFeedbackBuffer
    deriving ( Eq, Ord, Show )
 
 marshalBufferTarget :: BufferTarget -> GLenum
 marshalBufferTarget x = case x of
-   ArrayBuffer -> 0x8892
-   CopyReadBuffer -> 0x8F36
-   CopyWriteBuffer -> 0x8F37
-   ElementArrayBuffer -> 0x8893
-   PixelPackBuffer -> 0x88eb
-   PixelUnpackBuffer -> 0x88ec
+   ArrayBuffer -> gl_ARRAY_BUFFER
+   CopyReadBuffer -> gl_COPY_READ_BUFFER
+   CopyWriteBuffer -> gl_COPY_WRITE_BUFFER
+   ElementArrayBuffer -> gl_ELEMENT_ARRAY_BUFFER
+   PixelPackBuffer -> gl_PIXEL_PACK_BUFFER
+   PixelUnpackBuffer -> gl_PIXEL_UNPACK_BUFFER
+   TransformFeedbackBuffer -> gl_TRANSFORM_FEEDBACK_BUFFER
 
-bufferTargetToGetPName :: BufferTarget -> GetPName
+bufferTargetToGetPName :: BufferTarget -> PName1I
 bufferTargetToGetPName x = case x of
    ArrayBuffer -> GetArrayBufferBinding
    ElementArrayBuffer -> GetElementArrayBufferBinding
@@ -128,6 +112,7 @@ bufferTargetToGetPName x = case x of
    CopyWriteBuffer -> GetCopyWriteBuffer
    PixelPackBuffer -> GetPixelPackBufferBinding
    PixelUnpackBuffer -> GetPixelUnpackBufferBinding
+   TransformFeedbackBuffer -> GetTransformFeedbackBufferBinding
 
 --------------------------------------------------------------------------------
 
@@ -145,27 +130,27 @@ data BufferUsage =
 
 marshalBufferUsage :: BufferUsage -> GLenum
 marshalBufferUsage x = case x of
-   StreamDraw -> 0x88e0
-   StreamRead -> 0x88e1
-   StreamCopy -> 0x88e2
-   StaticDraw -> 0x88e4
-   StaticRead -> 0x88e5
-   StaticCopy -> 0x88e6
-   DynamicDraw -> 0x88e8
-   DynamicRead -> 0x88e9
-   DynamicCopy -> 0x88ea
+   StreamDraw -> gl_STREAM_DRAW
+   StreamRead -> gl_STREAM_READ
+   StreamCopy -> gl_STREAM_COPY
+   StaticDraw -> gl_STATIC_DRAW
+   StaticRead -> gl_STATIC_READ
+   StaticCopy -> gl_STATIC_COPY
+   DynamicDraw -> gl_DYNAMIC_DRAW
+   DynamicRead -> gl_DYNAMIC_READ
+   DynamicCopy -> gl_DYNAMIC_COPY
 
 unmarshalBufferUsage :: GLenum -> BufferUsage
 unmarshalBufferUsage x
-   | x == 0x88e0 = StreamDraw
-   | x == 0x88e1 = StreamRead
-   | x == 0x88e2 = StreamCopy
-   | x == 0x88e4 = StaticDraw
-   | x == 0x88e5 = StaticRead
-   | x == 0x88e6 = StaticCopy
-   | x == 0x88e8 = DynamicDraw
-   | x == 0x88e9 = DynamicRead
-   | x == 0x88ea = DynamicCopy
+   | x == gl_STREAM_DRAW = StreamDraw
+   | x == gl_STREAM_READ = StreamRead
+   | x == gl_STREAM_COPY = StreamCopy
+   | x == gl_STATIC_DRAW = StaticDraw
+   | x == gl_STATIC_READ = StaticRead
+   | x == gl_STATIC_COPY = StaticCopy
+   | x == gl_DYNAMIC_DRAW = DynamicDraw
+   | x == gl_DYNAMIC_READ = DynamicRead
+   | x == gl_DYNAMIC_COPY = DynamicCopy
    | otherwise = error ("unmarshalBufferUsage: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
@@ -178,15 +163,15 @@ data BufferAccess =
 
 marshalBufferAccess :: BufferAccess -> GLenum
 marshalBufferAccess x = case x of
-   ReadOnly -> 0x88b8
-   WriteOnly -> 0x88b9
-   ReadWrite -> 0x88ba
+   ReadOnly -> gl_READ_ONLY
+   WriteOnly -> gl_WRITE_ONLY
+   ReadWrite -> gl_READ_WRITE
 
 unmarshalBufferAccess :: GLenum -> BufferAccess
 unmarshalBufferAccess x
-   | x == 0x88b8 = ReadOnly
-   | x == 0x88b9 = WriteOnly
-   | x == 0x88ba = ReadWrite
+   | x == gl_READ_ONLY = ReadOnly
+   | x == gl_WRITE_ONLY = WriteOnly
+   | x == gl_READ_WRITE = ReadWrite
    | otherwise = error ("unmarshalBufferAccess: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
@@ -197,7 +182,7 @@ bindBuffer t = makeStateVar (getBindBuffer t) (setBindBuffer t)
 getBindBuffer :: BufferTarget -> IO (Maybe BufferObject)
 getBindBuffer = bufferQuery bufferTargetToGetPName
 
-bufferQuery :: (a -> GetPName) -> a -> IO (Maybe BufferObject)
+bufferQuery :: (a -> PName1I) -> a -> IO (Maybe BufferObject)
 bufferQuery func t = do
    buf <- getInteger1 (BufferObject . fromIntegral) (func t)
    return $ if buf == noBufferObject then Nothing else Just buf
@@ -207,11 +192,9 @@ noBufferObject = BufferObject 0
 
 setBindBuffer :: BufferTarget -> Maybe BufferObject -> IO ()
 setBindBuffer t =
-   glBindBufferARB (marshalBufferTarget t) . bufferID . maybe noBufferObject id
+   glBindBuffer (marshalBufferTarget t) . bufferID . fromMaybe noBufferObject
 
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glBindBufferARB,GLenum -> GLuint -> IO ())
-
-clientArrayTypeToGetPName :: ClientArrayType -> GetPName
+clientArrayTypeToGetPName :: ClientArrayType -> PName1I
 clientArrayTypeToGetPName x = case x of
    VertexArray -> GetVertexArrayBufferBinding
    NormalArray -> GetNormalArrayBufferBinding
@@ -250,9 +233,7 @@ getBufferData t = do
 
 setBufferData :: BufferTarget -> (GLsizeiptr, Ptr a, BufferUsage) -> IO ()
 setBufferData t (s, p, u) =
-   glBufferDataARB (marshalBufferTarget t) s p (marshalBufferUsage u)
-
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glBufferDataARB,GLenum -> GLsizeiptr -> Ptr a -> GLenum -> IO ())
+   glBufferData (marshalBufferTarget t) s p (marshalBufferUsage u)
 
 --------------------------------------------------------------------------------
 
@@ -263,12 +244,8 @@ data TransferDirection =
 
 bufferSubData ::
    BufferTarget -> TransferDirection -> GLintptr -> GLsizeiptr -> Ptr a -> IO ()
-bufferSubData t WriteToBuffer = glBufferSubDataARB (marshalBufferTarget t)
-bufferSubData t ReadFromBuffer = glGetBufferSubDataARB (marshalBufferTarget t)
-
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glBufferSubDataARB,GLenum -> GLintptr -> GLsizeiptr -> Ptr a -> IO ())
-
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glGetBufferSubDataARB,GLenum -> GLintptr -> GLsizeiptr -> Ptr a -> IO ())
+bufferSubData t WriteToBuffer = glBufferSubData (marshalBufferTarget t)
+bufferSubData t ReadFromBuffer = glGetBufferSubData (marshalBufferTarget t)
 
 --------------------------------------------------------------------------------
 
@@ -280,29 +257,24 @@ data GetBufferPName =
 
 marshalGetBufferPName :: GetBufferPName -> GLenum
 marshalGetBufferPName x = case x of
-   GetBufferSize -> 0x8764
-   GetBufferUsage -> 0x8765
-   GetBufferAccess -> 0x88bb
-   GetBufferMapped -> 0x88bc
+   GetBufferSize -> gl_BUFFER_SIZE
+   GetBufferUsage -> gl_BUFFER_USAGE
+   GetBufferAccess -> gl_BUFFER_ACCESS
+   GetBufferMapped -> gl_BUFFER_MAPPED
 
 getBufferParameter :: BufferTarget -> (GLenum -> a) -> GetBufferPName -> IO a
 getBufferParameter t f p = alloca $ \buf -> do
-   glGetBufferParameterivARB (marshalBufferTarget t)
-                             (marshalGetBufferPName p) buf
+   glGetBufferParameteriv (marshalBufferTarget t)
+                          (marshalGetBufferPName p) buf
    peek1 (f . fromIntegral) buf
 
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glGetBufferParameterivARB,GLenum -> GLenum -> Ptr GLint -> IO ())
-
 --------------------------------------------------------------------------------
 
 getBufferPointer :: BufferTarget -> IO (Ptr a)
 getBufferPointer t = alloca $ \buf -> do
-   -- only one pname: GL_BUFFER_MAP_POINTER
-   glGetBufferPointervARB (marshalBufferTarget t) 0x88bd buf
+   glGetBufferPointerv (marshalBufferTarget t) gl_BUFFER_MAP_POINTER buf
    peek buf
 
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glGetBufferPointervARB,GLenum -> GLenum -> Ptr (Ptr a) -> IO ())
-
 --------------------------------------------------------------------------------
 
 data MappingFailure =
@@ -326,14 +298,10 @@ mapBuffer :: BufferTarget -> BufferAccess -> IO (Maybe (Ptr a))
 mapBuffer t = fmap (maybeNullPtr Nothing Just) . mapBuffer_ t
 
 mapBuffer_ :: BufferTarget -> BufferAccess -> IO (Ptr a)
-mapBuffer_ t = glMapBufferARB (marshalBufferTarget t) . marshalBufferAccess
-
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glMapBufferARB,GLenum -> GLenum -> IO (Ptr a))
+mapBuffer_ t = glMapBuffer (marshalBufferTarget t) . marshalBufferAccess
 
 unmapBuffer :: BufferTarget -> IO Bool
-unmapBuffer = fmap unmarshalGLboolean . glUnmapBufferARB . marshalBufferTarget
-
-EXTENSION_ENTRY("GL_ARB_vertex_buffer_object or OpenGL 1.5",glUnmapBufferARB,GLenum -> IO GLboolean)
+unmapBuffer = fmap unmarshalGLboolean . glUnmapBuffer . marshalBufferTarget
 
 bufferAccess :: BufferTarget -> GettableStateVar BufferAccess
 bufferAccess t = makeGettableStateVar $
@@ -342,3 +310,111 @@ bufferAccess t = makeGettableStateVar $
 bufferMapped :: BufferTarget -> GettableStateVar Bool
 bufferMapped t = makeGettableStateVar $
    getBufferParameter t unmarshalGLboolean GetBufferMapped
+
+--------------------------------------------------------------------------------
+
+data BufferRangeAccessBit =
+     ReadBit
+   | WriteBit
+   | InvalidateRangeBit
+   | InvalidateBufferBit
+   | FlushExplicitBit
+   | UnsychronizedBit
+
+type Offset = GLintptr
+type Length = GLsizeiptr
+
+marshalBufferRangeAccessBit :: BufferRangeAccessBit -> GLenum
+marshalBufferRangeAccessBit x = case x of
+    ReadBit -> gl_MAP_READ_BIT
+    WriteBit -> gl_MAP_WRITE_BIT
+    InvalidateRangeBit -> gl_MAP_INVALIDATE_RANGE_BIT
+    InvalidateBufferBit -> gl_MAP_INVALIDATE_BUFFER_BIT
+    FlushExplicitBit -> gl_MAP_FLUSH_EXPLICIT_BIT
+    UnsychronizedBit -> gl_MAP_FLUSH_EXPLICIT_BIT
+
+marshalToBitfield :: [BufferRangeAccessBit] -> GLenum
+marshalToBitfield b = foldl1' (.|.)  $ map marshalBufferRangeAccessBit b
+
+--------------------------------------------------------------------------------
+
+mapBufferRange_ :: BufferTarget -> Offset -> Length ->
+   [BufferRangeAccessBit] -> IO (Ptr a)
+mapBufferRange_ t o l b = glMapBufferRange (marshalBufferTarget t) o l
+    (fromIntegral $ marshalToBitfield b)
+
+mapBufferRange :: BufferTarget -> Offset -> Length ->
+   [BufferRangeAccessBit] -> IO (Maybe (Ptr a))
+mapBufferRange t o l b = fmap (maybeNullPtr Nothing Just) $ mapBufferRange_ t o l b
+
+flushMappedBufferRange :: BufferTarget -> Offset -> Length -> IO()
+flushMappedBufferRange t = glFlushMappedBufferRange (marshalBufferTarget t)
+
+
+--------------------------------------------------------------------------------
+type BufferIndex = GLuint
+
+type RangeStartIndex = GLintptr
+type RangeSize = GLsizeiptr
+type BufferRange = (BufferObject, RangeStartIndex, RangeSize)
+
+data IndexedBufferTarget =
+     IndexedTransformFeedBackbuffer
+--marshaling
+marshalIndexedBufferTarget :: IndexedBufferTarget -> IPName1I
+marshalIndexedBufferTarget x = case x of
+   IndexedTransformFeedBackbuffer -> GetTransformFeedbackBuffer
+
+marshalIndexedBufferStart :: IndexedBufferTarget -> IPName1I
+marshalIndexedBufferStart x = case x of
+   IndexedTransformFeedBackbuffer -> GetTransformFeedbackBufferStart
+
+marshalIndexedBufferSize :: IndexedBufferTarget -> IPName1I
+marshalIndexedBufferSize x = case x of
+   IndexedTransformFeedBackbuffer -> GetTransformFeedbackBufferSize
+
+getIndexed :: Num a => IPName1I -> BufferIndex -> GettableStateVar a
+getIndexed e i = makeGettableStateVar $ getInteger1i fromIntegral e i
+
+--buffer
+bindBufferBase :: IndexedBufferTarget -> BufferIndex -> StateVar (Maybe BufferObject)
+bindBufferBase t i = makeStateVar (getIndexedBufferBinding t i) (setIndexedBufferBase t i)
+
+setIndexedBufferBase :: IndexedBufferTarget -> BufferIndex -> Maybe BufferObject -> IO ()
+setIndexedBufferBase t i buf=
+   case marshalGetPName . marshalIndexedBufferTarget $ t of
+      Nothing -> recordInvalidEnum
+      Just t' ->
+         glBindBufferBase t' i . bufferID . fromMaybe noBufferObject $ buf
+
+getIndexedBufferBinding :: IndexedBufferTarget -> BufferIndex -> IO (Maybe BufferObject)
+getIndexedBufferBinding t i = do
+   buf <- getInteger1i (BufferObject . fromIntegral) (marshalIndexedBufferTarget t) i
+   return $ if buf == noBufferObject then Nothing else Just buf
+
+bindBufferRange :: IndexedBufferTarget -> BufferIndex -> StateVar (Maybe BufferRange)
+bindBufferRange t i = makeStateVar (getIndexedBufferRange t i) (setIndexedBufferRange t i)
+
+setIndexedBufferRange :: IndexedBufferTarget -> BufferIndex -> Maybe BufferRange -> IO ()
+setIndexedBufferRange t i (Just (buf, start, range)) =
+   case marshalGetPName . marshalIndexedBufferTarget $ t of
+      Nothing -> recordInvalidEnum
+      Just t' -> glBindBufferRange t' i (bufferID buf) start range
+setIndexedBufferRange t i Nothing = setIndexedBufferBase t i Nothing
+
+getIndexedBufferRange :: IndexedBufferTarget -> BufferIndex -> IO(Maybe BufferRange)
+getIndexedBufferRange t i = do
+  buf <- getInteger1i (BufferObject . fromIntegral) (marshalIndexedBufferTarget t) i
+  if buf == noBufferObject
+     then return Nothing
+     else do
+        start <- get $ indexedBufferStart t i
+        size <- get $ indexedBufferSize t i
+        return $ Just (buf, start, size)
+
+
+indexedBufferStart :: IndexedBufferTarget -> BufferIndex -> GettableStateVar RangeStartIndex
+indexedBufferStart = getIndexed . marshalIndexedBufferStart
+
+indexedBufferSize :: IndexedBufferTarget -> BufferIndex -> GettableStateVar RangeSize
+indexedBufferSize = getIndexed . marshalIndexedBufferSize
