@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.Clipping
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -16,20 +16,15 @@ module Graphics.Rendering.OpenGL.GL.Clipping (
    ClipPlaneName(..), clipPlane, maxClipPlanes
 ) where
 
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Marshal.Utils ( with )
-import Foreign.Ptr ( Ptr, castPtr )
-import Graphics.Rendering.OpenGL.GL.BasicTypes ( GLenum, GLsizei, GLdouble )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   EnableCap(CapClipPlane), makeStateVarMaybe )
-import Graphics.Rendering.OpenGL.GL.CoordTrans ( Plane(..) )
-import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek1 )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetClipPlane,GetMaxClipPlanes),
-   clipPlaneIndexToEnum, getDoublev, getSizei1 )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar, StateVar )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal ( recordInvalidEnum )
+import Data.StateVar
+import Foreign.Marshal.Utils
+import Foreign.Ptr
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.CoordTrans
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility ( glClipPlane )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -42,14 +37,15 @@ clipPlane :: ClipPlaneName -> StateVar (Maybe (Plane GLdouble))
 clipPlane (ClipPlaneName i) =
    makeStateVarMaybe
       (return (CapClipPlane i))
-      (alloca $ \buf -> do
-          getDoublev (GetClipPlane i) (castPtr buf)
-          peek1 id (buf :: Ptr (Plane GLdouble)))
-      (\plane -> maybe recordInvalidEnum (with plane . glClipPlane)
+--      (alloca $ \buf -> do
+--          getDoublev (GetClipPlane i) (castPtr buf)
+--          peek1 id (buf :: Ptr (Plane GLdouble)))
+      (getDouble4 Plane (GetClipPlane i))
+      (\plane -> maybe recordInvalidEnum (with plane . glClipPlane_)
                        (clipPlaneIndexToEnum i))
 
-foreign import CALLCONV unsafe "glClipPlane" glClipPlane ::
-   GLenum -> Ptr (Plane GLdouble) -> IO ()
+glClipPlane_ :: GLenum -> Ptr (Plane GLdouble) -> IO ()
+glClipPlane_ plane ptr = glClipPlane plane (castPtr ptr)
 
 --------------------------------------------------------------------------------
 
