@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.Colors
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -36,38 +36,39 @@ module Graphics.Rendering.OpenGL.GL.Colors (
    ColorMaterialParameter(..), colorMaterial,
 
    -- * Flatshading
-   ShadingModel(..), shadeModel
+   ShadingModel(..), shadeModel,
+
+   -- * Color clamping
+   ClampTarget(..), ClampMode(..),
+   clampColor,
 ) where
 
-import Control.Monad ( liftM2, liftM3 )
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Marshal.Array ( allocaArray, withArray )
-import Foreign.Marshal.Utils ( with )
-import Foreign.Ptr ( Ptr )
-import Foreign.Storable ( Storable(peek) )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   marshalCapability, unmarshalCapability,
-   EnableCap(CapVertexProgramTwoSide,CapLighting,CapColorMaterial,CapLight),
-   makeCapability,
-   makeStateVarMaybe )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLenum, GLint, GLsizei, GLfloat,Capability )
-import Graphics.Rendering.OpenGL.GL.Face (
-   Face(..), marshalFace, unmarshalFace )
-import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek3 )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetMaxLights, GetFrontFace,GetShadeModel,
-            GetLightModelAmbient, GetLightModelLocalViewer,
-            GetLightModelTwoSide, GetLightModelColorControl,
-            GetColorMaterialFace,GetColorMaterialParameter,
-            GetMaxShininess,GetMaxSpotExponent),
-   getBoolean1, getEnum1, getSizei1, getFloat1, getFloat4, lightIndexToEnum )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   HasGetter(get), HasSetter(($=)),
-   GettableStateVar, makeGettableStateVar, StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.VertexSpec (
-   Color4(..), Normal3(..), Vertex4(..), Index1(..) )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal ( recordInvalidEnum )
+import Control.Monad
+import Data.StateVar
+import Data.Tensor
+import Foreign.Marshal.Alloc
+import Foreign.Marshal.Array
+import Foreign.Marshal.Utils
+import Foreign.Ptr
+import Foreign.Storable
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.Face
+import Graphics.Rendering.OpenGL.GL.PeekPoke
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.VertexSpec
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glColorMaterial, glGetLightfv, glGetMaterialfv, glGetMaterialiv,
+   glLightModelfv, glLightModeli, glLightfv, glMaterialfv, glMaterialiv,
+   glShadeModel, gl_AMBIENT, gl_AMBIENT_AND_DIFFUSE, gl_COLOR_INDEXES,
+   gl_CONSTANT_ATTENUATION, gl_DIFFUSE, gl_EMISSION, gl_FLAT,
+   gl_LIGHT_MODEL_AMBIENT, gl_LIGHT_MODEL_COLOR_CONTROL,
+   gl_LIGHT_MODEL_LOCAL_VIEWER, gl_LIGHT_MODEL_TWO_SIDE, gl_LINEAR_ATTENUATION,
+   gl_POSITION, gl_QUADRATIC_ATTENUATION, gl_SEPARATE_SPECULAR_COLOR,
+   gl_SHININESS, gl_SINGLE_COLOR, gl_SMOOTH, gl_SPECULAR, gl_SPOT_CUTOFF,
+   gl_SPOT_DIRECTION, gl_SPOT_EXPONENT,
+   gl_CLAMP_FRAGMENT_COLOR, gl_CLAMP_VERTEX_COLOR )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -99,13 +100,13 @@ data FrontFaceDirection =
 
 marshalFrontFaceDirection :: FrontFaceDirection -> GLenum
 marshalFrontFaceDirection x = case x of
-   CW -> 0x900
-   CCW -> 0x901
+   CW -> gl_CW
+   CCW -> gl_CCW
 
 unmarshalFrontFaceDirection :: GLenum -> FrontFaceDirection
 unmarshalFrontFaceDirection x
-   | x == 0x900 = CW
-   | x == 0x901 = CCW
+   | x == gl_CW = CW
+   | x == gl_CCW = CCW
    | otherwise = error ("unmarshalFrontFaceDirection: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
@@ -116,8 +117,6 @@ frontFace =
       (getEnum1 unmarshalFrontFaceDirection GetFrontFace)
       (glFrontFace . marshalFrontFaceDirection)
 
-foreign import CALLCONV unsafe "glFrontFace" glFrontFace :: GLenum -> IO ()
-
 --------------------------------------------------------------------------------
 
 data MaterialParameter =
@@ -131,13 +130,13 @@ data MaterialParameter =
 
 marshalMaterialParameter :: MaterialParameter -> GLenum
 marshalMaterialParameter x = case x of
-   MaterialEmission -> 0x1600
-   MaterialShininess -> 0x1601
-   MaterialAmbientAndDiffuse -> 0x1602
-   MaterialColorIndexes -> 0x1603
-   MaterialAmbient -> 0x1200
-   MaterialDiffuse -> 0x1201
-   MaterialSpecular -> 0x1202
+   MaterialEmission -> gl_EMISSION
+   MaterialShininess -> gl_SHININESS
+   MaterialAmbientAndDiffuse -> gl_AMBIENT_AND_DIFFUSE
+   MaterialColorIndexes -> gl_COLOR_INDEXES
+   MaterialAmbient -> gl_AMBIENT
+   MaterialDiffuse -> gl_DIFFUSE
+   MaterialSpecular -> gl_SPECULAR
 
 --------------------------------------------------------------------------------
 
@@ -171,11 +170,11 @@ makeMaterialVar getter setter materialParameter face =
    where mp = marshalMaterialParameter materialParameter
          f  = marshalFace face
 
-foreign import CALLCONV unsafe "glGetMaterialfv" glGetMaterialfvc ::
-   GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ()
+glGetMaterialfvc :: GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ()
+glGetMaterialfvc face pname ptr = glGetMaterialfv face pname (castPtr ptr)
 
-foreign import CALLCONV unsafe "glMaterialfv" glMaterialfvc ::
-   GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ()
+glMaterialfvc :: GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ()
+glMaterialfvc face pname ptr = glMaterialfv face pname (castPtr ptr)
 
 --------------------------------------------------------------------------------
 
@@ -183,11 +182,11 @@ materialShininess :: Face -> StateVar GLfloat
 materialShininess =
    makeMaterialVar glGetMaterialfvf glMaterialff MaterialShininess
 
-foreign import CALLCONV unsafe "glGetMaterialfv" glGetMaterialfvf ::
-   GLenum -> GLenum -> Ptr GLfloat -> IO ()
+glGetMaterialfvf :: GLenum -> GLenum -> Ptr GLfloat -> IO ()
+glGetMaterialfvf face pname ptr = glGetMaterialfv face pname (castPtr ptr)
 
-foreign import CALLCONV unsafe "glMaterialfv" glMaterialff ::
-   GLenum -> GLenum -> Ptr GLfloat -> IO ()
+glMaterialff :: GLenum -> GLenum -> Ptr GLfloat -> IO ()
+glMaterialff face pname ptr = glMaterialfv face pname (castPtr ptr)
 
 maxShininess :: GettableStateVar GLfloat
 maxShininess = makeGettableStateVar $ getFloat1 id GetMaxShininess
@@ -210,9 +209,6 @@ getMaterialColorIndexes face =
                       buf
       peek3 (\a d s -> (Index1 a, Index1 d, Index1 s)) buf
 
-foreign import CALLCONV unsafe "glGetMaterialiv" glGetMaterialiv ::
-   GLenum -> GLenum -> Ptr GLint -> IO ()
-
 setMaterialColorIndexes ::
    Face -> (Index1 GLint, Index1 GLint, Index1 GLint) -> IO ()
 setMaterialColorIndexes face (Index1 a, Index1 d, Index1 s) =
@@ -220,9 +216,6 @@ setMaterialColorIndexes face (Index1 a, Index1 d, Index1 s) =
       glMaterialiv (marshalFace face)
                    (marshalMaterialParameter MaterialColorIndexes)
 
-foreign import CALLCONV unsafe "glMaterialiv" glMaterialiv ::
-   GLenum -> GLenum -> Ptr GLint -> IO ()
-
 --------------------------------------------------------------------------------
 
 data LightParameter =
@@ -239,16 +232,16 @@ data LightParameter =
 
 marshalLightParameter :: LightParameter -> GLenum
 marshalLightParameter x = case x of
-   Ambient' -> 0x1200
-   Diffuse' -> 0x1201
-   Specular' -> 0x1202
-   Position -> 0x1203
-   SpotDirection -> 0x1204
-   SpotExponent -> 0x1205
-   SpotCutoff -> 0x1206
-   ConstantAttenuation -> 0x1207
-   LinearAttenuation -> 0x1208
-   QuadraticAttenuation -> 0x1209
+   Ambient' -> gl_AMBIENT
+   Diffuse' -> gl_DIFFUSE
+   Specular' -> gl_SPECULAR
+   Position -> gl_POSITION
+   SpotDirection -> gl_SPOT_DIRECTION
+   SpotExponent -> gl_SPOT_EXPONENT
+   SpotCutoff -> gl_SPOT_CUTOFF
+   ConstantAttenuation -> gl_CONSTANT_ATTENUATION
+   LinearAttenuation -> gl_LINEAR_ATTENUATION
+   QuadraticAttenuation -> gl_QUADRATIC_ATTENUATION
 
 --------------------------------------------------------------------------------
 
@@ -276,22 +269,22 @@ makeLightVar getter setter lightParameter defaultValue theLight =
          getLightVar = \l -> alloca $ \buf -> do getter l lp buf ; peek buf
          setLightVar = \val l -> with val $ setter l lp
 
-foreign import CALLCONV unsafe "glGetLightfv" glGetLightfvc ::
-   GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ()
+glGetLightfvc :: GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ()
+glGetLightfvc l pname ptr = glGetLightfv l pname (castPtr ptr)
 
-foreign import CALLCONV unsafe "glLightfv" glLightfvc ::
-   GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ()
+glLightfvc :: GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ()
+glLightfvc l pname ptr = glLightfv l pname (castPtr ptr)
 
 --------------------------------------------------------------------------------
 
 position :: Light -> StateVar (Vertex4 GLfloat)
 position = makeLightVar glGetLightfvv glLightfvv Position (Vertex4 0 0 0 0)
 
-foreign import CALLCONV unsafe "glLightfv" glLightfvv ::
-   GLenum -> GLenum -> Ptr (Vertex4 GLfloat) -> IO ()
+glLightfvv :: GLenum -> GLenum -> Ptr (Vertex4 GLfloat) -> IO ()
+glLightfvv l pname ptr = glLightfv l pname (castPtr ptr)
 
-foreign import CALLCONV unsafe "glGetLightfv" glGetLightfvv ::
-   GLenum -> GLenum -> Ptr (Vertex4 GLfloat) -> IO ()
+glGetLightfvv :: GLenum -> GLenum -> Ptr (Vertex4 GLfloat) -> IO ()
+glGetLightfvv l pname ptr = glGetLightfv l pname (castPtr ptr)
 
 --------------------------------------------------------------------------------
 
@@ -299,23 +292,17 @@ spotDirection :: Light -> StateVar (Normal3 GLfloat)
 spotDirection =
    makeLightVar glGetLightfvn glLightfvn SpotDirection (Normal3 0 0 0)
 
-foreign import CALLCONV unsafe "glLightfv" glLightfvn ::
-   GLenum -> GLenum -> Ptr (Normal3 GLfloat) -> IO ()
+glLightfvn :: GLenum -> GLenum -> Ptr (Normal3 GLfloat) -> IO ()
+glLightfvn l pname ptr = glLightfv l pname (castPtr ptr)
 
-foreign import CALLCONV unsafe "glGetLightfv" glGetLightfvn ::
-   GLenum -> GLenum -> Ptr (Normal3 GLfloat) -> IO ()
+glGetLightfvn :: GLenum -> GLenum -> Ptr (Normal3 GLfloat) -> IO ()
+glGetLightfvn l pname ptr = glGetLightfv l pname (castPtr ptr)
 
 --------------------------------------------------------------------------------
 
 spotExponent :: Light -> StateVar GLfloat
 spotExponent = makeLightVar glGetLightfv glLightfv SpotExponent 0
 
-foreign import CALLCONV unsafe "glLightfv" glLightfv ::
-   GLenum -> GLenum -> Ptr GLfloat -> IO ()
-
-foreign import CALLCONV unsafe "glGetLightfv" glGetLightfv ::
-   GLenum -> GLenum -> Ptr GLfloat -> IO ()
-
 maxSpotExponent :: GettableStateVar GLfloat
 maxSpotExponent = makeGettableStateVar $ getFloat1 id GetMaxSpotExponent
 
@@ -357,10 +344,10 @@ data LightModelParameter =
 
 marshalLightModelParameter :: LightModelParameter -> GLenum
 marshalLightModelParameter x = case x of
-   LightModelAmbient -> 0xb53
-   LightModelLocalViewer -> 0xb51
-   LightModelTwoSide -> 0xb52
-   LightModelColorControl -> 0x81f8
+   LightModelAmbient -> gl_LIGHT_MODEL_AMBIENT
+   LightModelLocalViewer -> gl_LIGHT_MODEL_LOCAL_VIEWER
+   LightModelTwoSide -> gl_LIGHT_MODEL_TWO_SIDE
+   LightModelColorControl -> gl_LIGHT_MODEL_COLOR_CONTROL
 
 --------------------------------------------------------------------------------
 
@@ -369,10 +356,7 @@ lightModelAmbient =
    makeStateVar
       (getFloat4 Color4 GetLightModelAmbient)
       (\c -> with c $
-                glLightModelfv (marshalLightModelParameter LightModelAmbient))
-
-foreign import CALLCONV unsafe "glLightModelfv" glLightModelfv ::
-   GLenum -> Ptr (Color4 GLfloat) -> IO ()
+                glLightModelfv (marshalLightModelParameter LightModelAmbient) . castPtr)
 
 --------------------------------------------------------------------------------
 
@@ -380,16 +364,13 @@ lightModelLocalViewer :: StateVar Capability
 lightModelLocalViewer =
    makeLightModelCapVar GetLightModelLocalViewer LightModelLocalViewer
 
-makeLightModelCapVar :: GetPName -> LightModelParameter -> StateVar Capability
+makeLightModelCapVar :: PName1I -> LightModelParameter -> StateVar Capability
 makeLightModelCapVar pname lightModelParameter =
    makeStateVar
       (getBoolean1 unmarshalCapability pname)
       (glLightModeli (marshalLightModelParameter lightModelParameter) .
                      fromIntegral . marshalCapability)
 
-foreign import CALLCONV unsafe "glLightModeli" glLightModeli ::
-   GLenum -> GLint -> IO ()
-
 --------------------------------------------------------------------------------
 
 lightModelTwoSide :: StateVar Capability
@@ -407,13 +388,13 @@ data LightModelColorControl =
 
 marshalLightModelColorControl :: LightModelColorControl -> GLenum
 marshalLightModelColorControl x = case x of
-   SingleColor -> 0x81f9
-   SeparateSpecularColor -> 0x81fa
+   SingleColor -> gl_SINGLE_COLOR
+   SeparateSpecularColor -> gl_SEPARATE_SPECULAR_COLOR
 
 unmarshalLightModelColorControl :: GLenum -> LightModelColorControl
 unmarshalLightModelColorControl x
-   | x == 0x81f9 = SingleColor
-   | x == 0x81fa = SeparateSpecularColor
+   | x == gl_SINGLE_COLOR = SingleColor
+   | x == gl_SEPARATE_SPECULAR_COLOR = SeparateSpecularColor
    | otherwise = error ("unmarshalLightModelColorControl: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
@@ -437,19 +418,19 @@ data ColorMaterialParameter =
 
 marshalColorMaterialParameter :: ColorMaterialParameter -> GLenum
 marshalColorMaterialParameter x = case x of
-   Ambient -> 0x1200
-   Diffuse -> 0x1201
-   Specular -> 0x1202
-   Emission -> 0x1600
-   AmbientAndDiffuse -> 0x1602
+   Ambient -> gl_AMBIENT
+   Diffuse -> gl_DIFFUSE
+   Specular -> gl_SPECULAR
+   Emission -> gl_EMISSION
+   AmbientAndDiffuse -> gl_AMBIENT_AND_DIFFUSE
 
 unmarshalColorMaterialParameter :: GLenum -> ColorMaterialParameter
 unmarshalColorMaterialParameter x
-   | x == 0x1200 = Ambient
-   | x == 0x1201 = Diffuse
-   | x == 0x1202 = Specular
-   | x == 0x1600 = Emission
-   | x == 0x1602 = AmbientAndDiffuse
+   | x == gl_AMBIENT = Ambient
+   | x == gl_DIFFUSE = Diffuse
+   | x == gl_SPECULAR = Specular
+   | x == gl_EMISSION = Emission
+   | x == gl_AMBIENT_AND_DIFFUSE = AmbientAndDiffuse
    | otherwise = error ("unmarshalColorMaterialParameter: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
@@ -465,9 +446,6 @@ colorMaterial =
       (\(face, param) -> glColorMaterial (marshalFace face)
                                          (marshalColorMaterialParameter param))
 
-foreign import CALLCONV unsafe "glColorMaterial" glColorMaterial ::
-   GLenum -> GLenum -> IO ()
-
 --------------------------------------------------------------------------------
 
 data ShadingModel =
@@ -477,13 +455,13 @@ data ShadingModel =
 
 marshalShadingModel :: ShadingModel -> GLenum
 marshalShadingModel x = case x of
-   Flat -> 0x1d00
-   Smooth -> 0x1d01
+   Flat -> gl_FLAT
+   Smooth -> gl_SMOOTH
 
 unmarshalShadingModel :: GLenum -> ShadingModel
 unmarshalShadingModel x
-   | x == 0x1d00 = Flat
-   | x == 0x1d01 = Smooth
+   | x == gl_FLAT = Flat
+   | x == gl_SMOOTH = Smooth
    | otherwise = error ("unmarshalShadingModel: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
@@ -494,4 +472,49 @@ shadeModel =
       (getEnum1 unmarshalShadingModel GetShadeModel)
       (glShadeModel . marshalShadingModel)
 
-foreign import CALLCONV unsafe "glShadeModel" glShadeModel :: GLenum -> IO ()
+--------------------------------------------------------------------------------
+
+data ClampTarget =
+     ClampVertexColor
+   | ClampFragmentColor
+   | ClampReadColor
+
+marshalClampTarget :: ClampTarget -> GLenum
+marshalClampTarget x = case x of
+   ClampVertexColor -> gl_CLAMP_VERTEX_COLOR
+   ClampFragmentColor -> gl_CLAMP_FRAGMENT_COLOR
+   ClampReadColor -> gl_CLAMP_READ_COLOR
+
+marshalClampTargetToPName :: ClampTarget -> PName1I
+marshalClampTargetToPName x = case x of
+   ClampFragmentColor -> GetFragmentColorClamp
+   ClampVertexColor -> GetVertexColorClamp
+   ClampReadColor -> GetReadColorClamp
+
+
+--------------------------------------------------------------------------------
+
+data ClampMode =
+     ClampOn
+   | FixedOnly
+   | ClampOff
+
+marshalClampMode :: ClampMode -> GLenum
+marshalClampMode x = case x of
+   ClampOn -> gl_TRUE
+   FixedOnly -> gl_FIXED_ONLY
+   ClampOff -> gl_FALSE
+
+unmarshalClampMode :: GLenum -> ClampMode
+unmarshalClampMode x
+   | x == gl_TRUE = ClampOn
+   | x == gl_FIXED_ONLY = FixedOnly
+   | x == gl_FALSE = ClampOff
+   | otherwise = error $ "unmarshalClampMode: unknown enum value " ++ show x
+
+--------------------------------------------------------------------------------
+
+clampColor :: ClampTarget -> StateVar ClampMode
+clampColor ct = makeStateVar (getClampColor ct) (setClampColor ct)
+   where setClampColor t = glClampColor (marshalClampTarget t) . marshalClampMode
+         getClampColor = getEnum1 unmarshalClampMode . marshalClampTargetToPName
