@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.CoordTrans
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -13,6 +13,8 @@
 --
 --------------------------------------------------------------------------------
 
+{-# LANGUAGE TypeSynonymInstances #-}
+
 module Graphics.Rendering.OpenGL.GL.CoordTrans (
    -- * Controlling the Viewport
    depthRange,
@@ -20,7 +22,6 @@ module Graphics.Rendering.OpenGL.GL.CoordTrans (
 
    -- * Matrices
    MatrixMode(..), matrixMode,
-   Vector1(..), Vector2(..), Vector3(..), Vector4(..),
    MatrixOrder(..), MatrixComponent(rotate,translate,scale), Matrix(..),
    currentMatrix, matrix, multMatrix, GLmatrix, loadIdentity,
    ortho, frustum, depthClamp,
@@ -35,49 +36,31 @@ module Graphics.Rendering.OpenGL.GL.CoordTrans (
    Plane(..), TextureCoordName(..), TextureGenMode(..), textureGenMode
 ) where
 
-import Foreign.ForeignPtr ( ForeignPtr, mallocForeignPtrArray, withForeignPtr )
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Marshal.Array ( allocaArray, peekArray, pokeArray, withArray )
-import Foreign.Marshal.Utils ( with )
-import Foreign.Ptr ( Ptr, castPtr )
-import Foreign.Storable ( Storable(..) )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   EnableCap(CapRescaleNormal, CapNormalize,CapDepthClamp,
-             CapTextureGenS, CapTextureGenT,
-             CapTextureGenR, CapTextureGenQ),
-   makeCapability, makeStateVarMaybe )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLenum, GLint, GLsizei, GLfloat, GLdouble, GLclampd, Capability(..) )
-import Graphics.Rendering.OpenGL.GL.Exception ( bracket, unsafeBracket_ )
-import Graphics.Rendering.OpenGL.GL.Extensions (
-   FunPtr, unsafePerformIO, Invoker, getProcAddress )
-import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek1, peek4, poke4 )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetDepthRange,GetViewport,GetMaxViewportDims,GetMatrixMode,
-            GetModelviewMatrix,GetProjectionMatrix,
-            GetTextureMatrix, GetColorMatrix,GetMatrixPalette,GetActiveTexture,
-            GetCurrentMatrixStackDepth,GetModelviewStackDepth,
-            GetMaxModelviewStackDepth,GetProjectionStackDepth,
-            GetMaxProjectionStackDepth,GetTextureStackDepth,
-            GetMaxTextureStackDepth,GetColorMatrixStackDepth,
-            GetMaxColorMatrixStackDepth,GetMaxMatrixPaletteStackDepth),
-   getInteger2, getInteger4, getEnum1, getSizei1, getFloatv,
-   getDouble2, getDoublev, modelviewIndexToEnum, modelviewEnumToIndex )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar, HasGetter(get), HasSetter(($=)),
-   StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.Tensor (
-   Vector1(..), Vector2(..), Vector3(..), Vector4(..) )
-import Graphics.Rendering.OpenGL.GL.Texturing.TextureUnit (
-   marshalTextureUnit, unmarshalTextureUnit )
-import Graphics.Rendering.OpenGL.GL.VertexSpec ( TextureUnit )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal (
-   recordInvalidEnum, recordInvalidValue )
-
---------------------------------------------------------------------------------
-
-#include "HsOpenGLExt.h"
-#include "HsOpenGLTypes.h"
+import Data.StateVar
+import Data.Tensor
+import Foreign.ForeignPtr
+import Foreign.Marshal.Alloc
+import Foreign.Marshal.Array
+import Foreign.Marshal.Utils
+import Foreign.Ptr
+import Foreign.Storable
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.Exception
+import Graphics.Rendering.OpenGL.GL.PeekPoke
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.Texturing.TextureUnit
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glFrustum, glGetTexGendv, glGetTexGeniv, glLoadIdentity, glLoadMatrixd,
+   glLoadMatrixf, glLoadTransposeMatrixd, glLoadTransposeMatrixf, glMatrixMode,
+   glMultMatrixd, glMultMatrixf, glMultTransposeMatrixd, glMultTransposeMatrixf,
+   glOrtho, glPopMatrix, glPushMatrix, glRotated, glRotatef, glScaled, glScalef,
+   glTexGendv, glTexGeni, glTranslated, glTranslatef, gl_EYE_LINEAR,
+   gl_EYE_PLANE, gl_NORMAL_MAP, gl_OBJECT_LINEAR, gl_OBJECT_PLANE,
+   gl_PROJECTION, gl_Q, gl_R, gl_REFLECTION_MAP, gl_S, gl_SPHERE_MAP, gl_T,
+   gl_TEXTURE_GEN_MODE )
+import Graphics.Rendering.OpenGL.Raw.ARB.MatrixPalette ( gl_MATRIX_PALETTE )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -96,10 +79,7 @@ import Graphics.Rendering.OpenGL.GLU.ErrorsInternal (
 -- mappings such as (1, 0) are acceptable.
 
 depthRange :: StateVar (GLclampd, GLclampd)
-depthRange = makeStateVar (getDouble2 (,) GetDepthRange) (uncurry glDepthRange)
-
-foreign import CALLCONV unsafe "glDepthRange" glDepthRange ::
-   GLclampd -> GLclampd -> IO ()
+depthRange = makeStateVar (getClampd2 (,) GetDepthRange) (uncurry glDepthRange)
 
 --------------------------------------------------------------------------------
 
@@ -133,13 +113,10 @@ viewport = makeStateVar (getInteger4 makeVp GetViewport)
                         (\(Position x y, Size w h) -> glViewport x y w h)
    where makeVp x y w h = (Position x y, Size (fromIntegral w) (fromIntegral h))
 
-foreign import CALLCONV unsafe "glViewport" glViewport ::
-   GLint -> GLint -> GLsizei -> GLsizei -> IO ()
-
 -- | The implementation-dependent maximum viewport width and height.
 
 maxViewportDims :: GettableStateVar Size
-maxViewportDims = makeGettableStateVar (getInteger2 Size GetMaxViewportDims)
+maxViewportDims = makeGettableStateVar (getSizei2 Size GetMaxViewportDims)
 
 --------------------------------------------------------------------------------
 
@@ -156,23 +133,23 @@ data MatrixMode =
 marshalMatrixMode :: MatrixMode -> Maybe GLenum
 marshalMatrixMode x = case x of
    Modelview i -> modelviewIndexToEnum i
-   Projection -> Just 0x1701
-   Texture -> Just 0x1702
-   Color -> Just 0x1800
-   MatrixPalette -> Just 0x8840
+   Projection -> Just gl_PROJECTION
+   Texture -> Just gl_TEXTURE
+   Color -> Just gl_COLOR
+   MatrixPalette -> Just gl_MATRIX_PALETTE
 
 unmarshalMatrixMode :: GLenum -> MatrixMode
 unmarshalMatrixMode x
-   | x == 0x1701 = Projection
-   | x == 0x1702 = Texture
-   | x == 0x1800 = Color
-   | x == 0x8840 = MatrixPalette
+   | x == gl_PROJECTION = Projection
+   | x == gl_TEXTURE = Texture
+   | x == gl_COLOR = Color
+   | x == gl_MATRIX_PALETTE = MatrixPalette
    | otherwise =
         case modelviewEnumToIndex x of
            Just i -> Modelview i
            Nothing -> error ("unmarshalMatrixMode: illegal value " ++ show x)
 
-matrixModeToGetMatrix :: MatrixMode -> GetPName
+matrixModeToGetMatrix :: MatrixMode -> PNameMatrix
 matrixModeToGetMatrix x = case x of
    Modelview _   -> GetModelviewMatrix -- ???
    Projection    -> GetProjectionMatrix
@@ -180,7 +157,7 @@ matrixModeToGetMatrix x = case x of
    Color         -> GetColorMatrix
    MatrixPalette -> GetMatrixPalette
 
-matrixModeToGetStackDepth :: MatrixMode -> GetPName
+matrixModeToGetStackDepth :: MatrixMode -> PName1I
 matrixModeToGetStackDepth x =  case x of
    Modelview _   -> GetModelviewStackDepth
    Projection    -> GetProjectionStackDepth
@@ -188,7 +165,7 @@ matrixModeToGetStackDepth x =  case x of
    Color         -> GetColorMatrixStackDepth
    MatrixPalette -> error "matrixModeToGetStackDepth: impossible"
 
-matrixModeToGetMaxStackDepth :: MatrixMode -> GetPName
+matrixModeToGetMaxStackDepth :: MatrixMode -> PName1I
 matrixModeToGetMaxStackDepth x = case x of
    Modelview _    -> GetMaxModelviewStackDepth
    Projection     -> GetMaxProjectionStackDepth
@@ -206,8 +183,6 @@ matrixMode =
    makeStateVar (getEnum1 unmarshalMatrixMode GetMatrixMode)
                 (maybe recordInvalidValue glMatrixMode . marshalMatrixMode)
 
-foreign import CALLCONV unsafe "glMatrixMode" glMatrixMode :: GLenum -> IO ()
-
 --------------------------------------------------------------------------------
 
 data MatrixOrder = ColumnMajor | RowMajor
@@ -216,7 +191,7 @@ data MatrixOrder = ColumnMajor | RowMajor
 --------------------------------------------------------------------------------
 
 class Storable c => MatrixComponent c where
-   getMatrix :: GetPName -> Ptr c -> IO ()
+   getMatrix :: GetPNameMatrix p => p -> Ptr c -> IO ()
    loadMatrix :: Ptr c -> IO ()
    loadTransposeMatrix :: Ptr c -> IO ()
    multMatrix_ :: Ptr c -> IO ()
@@ -225,51 +200,28 @@ class Storable c => MatrixComponent c where
    translate :: Vector3 c -> IO ()
    scale :: c -> c -> c -> IO ()
 
-instance MatrixComponent GLfloat_ where
-   getMatrix = getFloatv
+instance MatrixComponent GLfloat where
+   getMatrix = getMatrixf
    loadMatrix = glLoadMatrixf
-   loadTransposeMatrix = glLoadTransposeMatrixfARB
+   loadTransposeMatrix = glLoadTransposeMatrixf
    multMatrix_ = glMultMatrixf
-   multTransposeMatrix = glMultTransposeMatrixfARB
+   multTransposeMatrix = glMultTransposeMatrixf
    rotate a (Vector3 x y z) = glRotatef a x y z
    translate (Vector3 x y z) = glTranslatef x y z
    scale = glScalef
 
-instance MatrixComponent GLdouble_ where
-   getMatrix = getDoublev
+instance MatrixComponent GLdouble where
+   getMatrix = getMatrixd
    loadMatrix = glLoadMatrixd
-   loadTransposeMatrix = glLoadTransposeMatrixdARB
+   loadTransposeMatrix = glLoadTransposeMatrixd
    multMatrix_ = glMultMatrixd
-   multTransposeMatrix = glMultTransposeMatrixdARB
+   multTransposeMatrix = glMultTransposeMatrixd
    rotate a (Vector3 x y z) = glRotated a x y z
    translate (Vector3 x y z) = glTranslated x y z
    scale = glScaled
 
 --------------------------------------------------------------------------------
 
-foreign import CALLCONV unsafe "glLoadMatrixf" glLoadMatrixf :: Ptr GLfloat -> IO ()
-foreign import CALLCONV unsafe "glLoadMatrixd" glLoadMatrixd :: Ptr GLdouble -> IO ()
-
-EXTENSION_ENTRY("GL_ARB_transpose_matrix or OpenGL 1.3",glLoadTransposeMatrixfARB,Ptr GLfloat -> IO ())
-EXTENSION_ENTRY("GL_ARB_transpose_matrix or OpenGL 1.3",glLoadTransposeMatrixdARB,Ptr GLdouble -> IO ())
-
-foreign import CALLCONV unsafe "glMultMatrixf" glMultMatrixf :: Ptr GLfloat -> IO ()
-foreign import CALLCONV unsafe "glMultMatrixd" glMultMatrixd :: Ptr GLdouble -> IO ()
-
-EXTENSION_ENTRY("GL_ARB_transpose_matrix or OpenGL 1.3",glMultTransposeMatrixfARB,Ptr GLfloat -> IO ())
-EXTENSION_ENTRY("GL_ARB_transpose_matrix or OpenGL 1.3",glMultTransposeMatrixdARB,Ptr GLdouble -> IO ())
-
-foreign import CALLCONV unsafe "glRotatef" glRotatef :: GLfloat -> GLfloat -> GLfloat -> GLfloat -> IO ()
-foreign import CALLCONV unsafe "glRotated" glRotated :: GLdouble -> GLdouble -> GLdouble -> GLdouble -> IO ()
-
-foreign import CALLCONV unsafe "glTranslatef" glTranslatef :: GLfloat -> GLfloat -> GLfloat -> IO ()
-foreign import CALLCONV unsafe "glTranslated" glTranslated :: GLdouble -> GLdouble -> GLdouble -> IO ()
-
-foreign import CALLCONV unsafe "glScalef" glScalef :: GLfloat -> GLfloat -> GLfloat -> IO ()
-foreign import CALLCONV unsafe "glScaled" glScaled :: GLdouble -> GLdouble -> GLdouble -> IO ()
-
---------------------------------------------------------------------------------
-
 class Matrix m where
    -- | Create a new matrix of the given order (containing undefined elements)
    -- and call the action to fill it with 4x4 elements.
@@ -324,7 +276,7 @@ withMatrixMode mode act =
       matrixMode $= mode
       act
 
-getMatrix' :: (Matrix m, MatrixComponent c) => GetPName -> IO (m c)
+getMatrix' :: (Matrix m, MatrixComponent c) => PNameMatrix -> IO (m c)
 getMatrix' = withNewMatrix ColumnMajor . getMatrix
 
 setMatrix :: (Matrix m, MatrixComponent c) => m c -> IO ()
@@ -344,14 +296,7 @@ multMatrix mat =
 --------------------------------------------------------------------------------
 
 data GLmatrix a = GLmatrix MatrixOrder (ForeignPtr a)
-#ifdef __HADDOCK__
--- Help Haddock a bit, because it doesn't do any instance inference.
-instance Eq (GLmatrix a)
-instance Ord (GLmatrix a)
-instance Show (GLmatrix a)
-#else
    deriving ( Eq, Ord, Show )
-#endif
 
 instance Matrix GLmatrix where
    withNewMatrix order f = do
@@ -363,15 +308,16 @@ instance Matrix GLmatrix where
 
 --------------------------------------------------------------------------------
 
-foreign import CALLCONV unsafe "glLoadIdentity" loadIdentity :: IO ()
+loadIdentity :: IO ()
+loadIdentity = glLoadIdentity
 
 --------------------------------------------------------------------------------
 
-foreign import CALLCONV unsafe "glOrtho" ortho ::
-   GLdouble -> GLdouble -> GLdouble -> GLdouble -> GLdouble -> GLdouble -> IO ()
+ortho :: GLdouble -> GLdouble -> GLdouble -> GLdouble -> GLdouble -> GLdouble -> IO ()
+ortho = glOrtho
 
-foreign import CALLCONV unsafe "glFrustum" frustum ::
-   GLdouble -> GLdouble -> GLdouble -> GLdouble -> GLdouble -> GLdouble -> IO ()
+frustum :: GLdouble -> GLdouble -> GLdouble -> GLdouble -> GLdouble -> GLdouble -> IO ()
+frustum = glFrustum
 
 --------------------------------------------------------------------------------
 
@@ -382,9 +328,7 @@ depthClamp = makeCapability CapDepthClamp
 
 activeTexture :: StateVar TextureUnit
 activeTexture = makeStateVar (getEnum1 unmarshalTextureUnit GetActiveTexture)
-                             (glActiveTextureARB . marshalTextureUnit)
-
-EXTENSION_ENTRY("GL_ARB_multitexture or OpenGL 1.3",glActiveTextureARB,GLenum -> IO ())   
+                             (glActiveTexture . marshalTextureUnit)
 
 --------------------------------------------------------------------------------
 
@@ -409,10 +353,6 @@ preservingMatrixMode = bracket (getEnum1 id GetMatrixMode) glMatrixMode . const
 unsafePreservingMatrix :: IO a -> IO a
 unsafePreservingMatrix = unsafeBracket_ glPushMatrix glPopMatrix
 
-foreign import CALLCONV unsafe "glPushMatrix" glPushMatrix :: IO ()
-
-foreign import CALLCONV unsafe "glPopMatrix" glPopMatrix :: IO ()
-
 --------------------------------------------------------------------------------
 
 stackDepth :: Maybe MatrixMode -> GettableStateVar GLsizei
@@ -468,10 +408,10 @@ data TextureCoordName =
 
 marshalTextureCoordName :: TextureCoordName -> GLenum
 marshalTextureCoordName x = case x of
-   S -> 0x2000
-   T -> 0x2001
-   R -> 0x2002
-   Q -> 0x2003
+   S -> gl_S
+   T -> gl_T
+   R -> gl_R
+   Q -> gl_Q
 
 --------------------------------------------------------------------------------
 
@@ -482,9 +422,9 @@ data TextureGenParameter =
 
 marshalTextureGenParameter :: TextureGenParameter -> GLenum
 marshalTextureGenParameter x = case x of
-   TextureGenMode -> 0x2500
-   ObjectPlane -> 0x2501
-   EyePlane -> 0x2502
+   TextureGenMode -> gl_TEXTURE_GEN_MODE
+   ObjectPlane -> gl_OBJECT_PLANE
+   EyePlane -> gl_EYE_PLANE
 
 --------------------------------------------------------------------------------
 
@@ -496,21 +436,22 @@ data TextureGenMode' =
    | ReflectionMap'
 
 marshalTextureGenMode' :: TextureGenMode' -> GLint
-marshalTextureGenMode' x = case x of
-   EyeLinear' -> 0x2400
-   ObjectLinear' -> 0x2401
-   SphereMap' -> 0x2402
-   NormalMap' -> 0x8511
-   ReflectionMap' -> 0x8512
+marshalTextureGenMode' x = fromIntegral $ case x of
+   EyeLinear' -> gl_EYE_LINEAR
+   ObjectLinear' -> gl_OBJECT_LINEAR
+   SphereMap' -> gl_SPHERE_MAP
+   NormalMap' -> gl_NORMAL_MAP
+   ReflectionMap' -> gl_REFLECTION_MAP
 
 unmarshalTextureGenMode' :: GLint -> TextureGenMode'
 unmarshalTextureGenMode' x
-   | x == 0x2400 = EyeLinear'
-   | x == 0x2401 = ObjectLinear'
-   | x == 0x2402 = SphereMap'
-   | x == 0x8511 = NormalMap'
-   | x == 0x8512 = ReflectionMap'
+   | y == gl_EYE_LINEAR = EyeLinear'
+   | y == gl_OBJECT_LINEAR = ObjectLinear'
+   | y == gl_SPHERE_MAP = SphereMap'
+   | y == gl_NORMAL_MAP = NormalMap'
+   | y == gl_REFLECTION_MAP = ReflectionMap'
    | otherwise = error ("unmarshalTextureGenMode': illegal value " ++ show x)
+   where y = fromIntegral x
 
 --------------------------------------------------------------------------------
 
@@ -568,36 +509,24 @@ getMode coord = alloca $ \buf -> do
                  buf
    peek1 unmarshalTextureGenMode' buf
 
-foreign import CALLCONV unsafe "glGetTexGeniv" glGetTexGeniv ::
-   GLenum -> GLenum -> Ptr GLint -> IO ()
-
 setMode :: TextureCoordName -> TextureGenMode -> IO ()
 setMode coord mode =
    glTexGeni (marshalTextureCoordName coord)
              (marshalTextureGenParameter TextureGenMode)
              (marshalTextureGenMode mode)
 
-foreign import CALLCONV unsafe "glTexGeni" glTexGeni ::
-    GLenum -> GLenum -> GLint -> IO ()
-
 --------------------------------------------------------------------------------
 
 getPlane :: TextureCoordName -> TextureGenParameter -> IO (Plane GLdouble)
 getPlane coord param = alloca $ \planeBuffer -> do
    glGetTexGendv (marshalTextureCoordName coord)
                  (marshalTextureGenParameter param)
-                 planeBuffer
+                 (castPtr planeBuffer)
    peek planeBuffer
 
-foreign import CALLCONV unsafe "glGetTexGendv" glGetTexGendv ::
-   GLenum -> GLenum -> Ptr (Plane GLdouble) -> IO ()
-
 setPlane :: TextureCoordName -> TextureGenParameter -> Plane GLdouble -> IO ()
 setPlane coord param plane =
    with plane $ \planeBuffer ->
       glTexGendv (marshalTextureCoordName coord)
                  (marshalTextureGenParameter param)
-                 planeBuffer
-
-foreign import CALLCONV unsafe "glTexGendv" glTexGendv ::
-   GLenum -> GLenum -> Ptr (Plane GLdouble) -> IO ()
+                 (castPtr planeBuffer)
