@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.DisplayLists
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -25,20 +25,18 @@ module Graphics.Rendering.OpenGL.GL.DisplayLists (
    genLists, deleteLists, isList,
 ) where
 
-import Foreign.Ptr ( Ptr )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLboolean, GLuint, GLsizei, GLenum )
-import Graphics.Rendering.OpenGL.GL.BufferObjects ( ObjectName(..) )
-import Graphics.Rendering.OpenGL.GL.DataType ( marshalDataType )
-import Graphics.Rendering.OpenGL.GL.VertexArrays ( DataType )
-import Graphics.Rendering.OpenGL.GL.Exception ( bracket_ )
-import Graphics.Rendering.OpenGL.GL.GLboolean ( unmarshalGLboolean )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetListIndex,GetListMode,GetMaxListNesting,GetListBase),
-   getEnum1, getSizei1 )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar, StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal ( recordOutOfMemory )
+import Data.ObjectName
+import Data.StateVar
+import Foreign.Ptr
+import Graphics.Rendering.OpenGL.GL.DataType
+import Graphics.Rendering.OpenGL.GL.Exception
+import Graphics.Rendering.OpenGL.GL.GLboolean
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+ glCallList, glCallLists, glDeleteLists, glEndList, glGenLists, glIsList,
+ glListBase, glNewList, gl_COMPILE, gl_COMPILE_AND_EXECUTE )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -64,8 +62,6 @@ genLists_ n = do
               return []
       else return [ DisplayList l | l <- [ first .. first + fromIntegral n - 1 ] ]
 
-foreign import CALLCONV unsafe "glGenLists" glGenLists :: GLsizei -> IO GLuint
-
 --------------------------------------------------------------------------------
 
 {-# DEPRECATED deleteLists "use `deleteObjectNames' instead" #-}
@@ -75,12 +71,9 @@ deleteLists = deleteLists_
 deleteLists_ :: [DisplayList] -> IO ()
 deleteLists_ = mapM_ (uncurry glDeleteLists) . combineConsecutive
 
-foreign import CALLCONV unsafe "glDeleteLists" glDeleteLists ::
-   DisplayList -> GLsizei -> IO ()
-
-combineConsecutive :: [DisplayList] -> [(DisplayList, GLsizei)]
+combineConsecutive :: [DisplayList] -> [(GLuint, GLsizei)]
 combineConsecutive []     = []
-combineConsecutive (z:zs) = (z, len) : combineConsecutive rest
+combineConsecutive (z@(DisplayList dl) :zs) = (dl, len) : combineConsecutive rest
    where (len, rest) = run (0 :: GLsizei) z zs
          run n x xs = case n + 1 of
                          m -> case xs of
@@ -96,10 +89,7 @@ isList :: DisplayList -> IO Bool
 isList = isList_
 
 isList_ :: DisplayList -> IO Bool
-isList_ = fmap unmarshalGLboolean . glIsList
-
-foreign import CALLCONV unsafe "glIsList" glIsList ::
-   DisplayList -> IO GLboolean
+isList_ (DisplayList dl) = fmap unmarshalGLboolean (glIsList dl)
 
 --------------------------------------------------------------------------------
 
@@ -110,24 +100,19 @@ data ListMode =
 
 marshalListMode :: ListMode -> GLenum
 marshalListMode x = case x of
-   Compile -> 0x1300
-   CompileAndExecute -> 0x1301
+   Compile -> gl_COMPILE
+   CompileAndExecute -> gl_COMPILE_AND_EXECUTE
 
 unmarshalListMode :: GLenum -> ListMode
 unmarshalListMode x
-   | x == 0x1300 = Compile
-   | x == 0x1301 = CompileAndExecute
+   | x == gl_COMPILE = Compile
+   | x == gl_COMPILE_AND_EXECUTE = CompileAndExecute
    | otherwise = error ("unmarshalListMode: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
 
 defineList :: DisplayList -> ListMode -> IO a -> IO a
-defineList lst mode = bracket_ (glNewList lst (marshalListMode mode)) glEndList
-
-foreign import CALLCONV unsafe "glNewList" glNewList ::
-   DisplayList -> GLenum -> IO ()
-
-foreign import CALLCONV unsafe "glEndList" glEndList :: IO ()
+defineList (DisplayList dl) mode = bracket_ (glNewList dl (marshalListMode mode)) glEndList
 
 defineNewList :: ListMode -> IO a -> IO DisplayList
 defineNewList mode action = do
@@ -136,7 +121,7 @@ defineNewList mode action = do
       then do recordOutOfMemory
               return noDisplayList
       else do let lst = head lists
-              defineList lst mode action
+              _ <- defineList lst mode action
               return lst
 
 --------------------------------------------------------------------------------
@@ -144,7 +129,7 @@ defineNewList mode action = do
 listIndex :: GettableStateVar (Maybe DisplayList)
 listIndex =
    makeGettableStateVar
-      (do l <- getEnum1 DisplayList GetListIndex
+      (do l <- getEnum1 (DisplayList . fromIntegral) GetListIndex
           return $ if l == noDisplayList then Nothing else Just l)
 
 noDisplayList :: DisplayList
@@ -158,17 +143,16 @@ maxListNesting = makeGettableStateVar (getSizei1 id GetMaxListNesting)
 
 --------------------------------------------------------------------------------
 
-foreign import CALLCONV unsafe "glCallList" callList :: DisplayList -> IO ()
+callList :: DisplayList -> IO ()
+callList (DisplayList dl) = glCallList dl
 
 callLists :: GLsizei -> DataType -> Ptr a -> IO ()
 callLists n = glCallLists n . marshalDataType
 
-foreign import CALLCONV unsafe "glCallLists" glCallLists ::
-   GLsizei -> GLenum -> Ptr a -> IO ()
-
 --------------------------------------------------------------------------------
 
 listBase :: StateVar DisplayList
-listBase = makeStateVar (getEnum1 DisplayList GetListBase) glListBase
-
-foreign import CALLCONV unsafe "glListBase" glListBase :: DisplayList -> IO ()
+listBase =
+   makeStateVar
+      (getEnum1 (DisplayList . fromIntegral) GetListBase)
+       (\(DisplayList dl) -> glListBase dl)
