@@ -1,9 +1,10 @@
+{-# LANGUAGE KindSignatures #-}
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.Evaluators
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -43,44 +44,36 @@ module Graphics.Rendering.OpenGL.GL.Evaluators (
    autoNormal
 ) where
 
-import Control.Monad ( zipWithM_ )
-import Data.List ( genericLength )
-import Foreign.ForeignPtr ( ForeignPtr, mallocForeignPtrArray, withForeignPtr )
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Marshal.Array ( allocaArray )
-import Foreign.Ptr ( Ptr, plusPtr )
-import Foreign.Storable ( Storable(peek,sizeOf) )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   EnableCap(CapAutoNormal), makeCapability, makeStateVarMaybe )
+import Control.Monad
+import Data.List
+import Data.StateVar
+import Foreign.ForeignPtr
+import Foreign.Marshal.Alloc
+import Foreign.Marshal.Array
+import Foreign.Ptr
+import Foreign.Storable
+import Graphics.Rendering.OpenGL.GL.Capability
 import Graphics.Rendering.OpenGL.GL.ControlPoint
 import Graphics.Rendering.OpenGL.GL.Domain
-import Graphics.Rendering.OpenGL.GL.BasicTypes ( GLenum, GLint, Capability )
-import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek2, peek4 )
-import Graphics.Rendering.OpenGL.GL.PolygonMode ( marshalPolygonMode )
-import Graphics.Rendering.OpenGL.GL.Polygons ( PolygonMode )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetMaxEvalOrder,
-            GetMap1GridSegments,GetMap1GridDomain,
-            GetMap2GridSegments,GetMap2GridDomain),
-   getSizei1, getInteger1, getInteger2 )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar, StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.VertexArrays ( NumComponents, Stride )
-
---------------------------------------------------------------------------------
-
-#include "HsOpenGLTypes.h"
+import Graphics.Rendering.OpenGL.GL.PeekPoke
+import Graphics.Rendering.OpenGL.GL.PolygonMode
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.VertexArrays
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glEvalMesh1, glEvalMesh2, glEvalPoint1, glEvalPoint2, glGetMapiv, gl_COEFF,
+   gl_DOMAIN, gl_ORDER )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
 type Order = GLint
 
 maxOrder :: GettableStateVar Order
-maxOrder = makeGettableStateVar (getSizei1 id GetMaxEvalOrder)
+maxOrder = makeGettableStateVar (getInteger1 id GetMaxEvalOrder)
 
 --------------------------------------------------------------------------------
 
-data Domain d => MapDescriptor d =
+data MapDescriptor d =
    MapDescriptor (d, d) Stride Order NumComponents
    deriving ( Eq, Ord, Show )
 
@@ -174,16 +167,9 @@ class Map1 m where
 
 --------------------------------------------------------------------------------
 
-data (ControlPoint c, Domain d) => GLmap1 c d =
+data GLmap1 (c :: * -> *) d =
    GLmap1 (MapDescriptor d) (ForeignPtr d)
-#ifdef __HADDOCK__
--- Help Haddock a bit, because it doesn't do any instance inference.
-instance Eq d => Eq (GLmap1 c d)
-instance Ord d => Ord (GLmap1 c d)
-instance Show d => Show (GLmap1 c d)
-#else
    deriving ( Eq, Ord, Show )
-#endif
 
 instance Map1 GLmap1 where
    withNewMap1 descriptor act = do
@@ -195,7 +181,7 @@ instance Map1 GLmap1 where
       withForeignPtr fp $ act descriptor
 
 --------------------------------------------------------------------------------
- 
+
 map1 :: (Map1 m, ControlPoint c, Domain d) => StateVar (Maybe (m c d))
 map1 = makeMap1StateVar enableCap1 getMap1 setMap1
 
@@ -281,16 +267,9 @@ class Map2 m where
 
 --------------------------------------------------------------------------------
 
-data (ControlPoint c, Domain d) => GLmap2 c d =
+data GLmap2 (c :: * -> *) d =
    GLmap2 (MapDescriptor d)  (MapDescriptor d) (ForeignPtr d)
-#ifdef __HADDOCK__
--- Help Haddock a bit, because it doesn't do any instance inference.
-instance Eq d => Eq (GLmap2 c d)
-instance Ord d => Ord (GLmap2 c d)
-instance Show d => Show (GLmap2 c d)
-#else
    deriving ( Eq, Ord, Show )
-#endif
 
 instance Map2 GLmap2 where
    withNewMap2 uDescriptor vDescriptor act = do
@@ -347,14 +326,9 @@ data GetMapQuery =
 
 marshalGetMapQuery :: GetMapQuery -> GLenum
 marshalGetMapQuery x = case x of
-   Coeff -> 0xa00
-   Order -> 0xa01
-   Domain -> 0xa02
-
---------------------------------------------------------------------------------
-
-foreign import CALLCONV unsafe "glGetMapiv" glGetMapiv ::
-   GLenum -> GLenum -> Ptr GLint -> IO ()
+   Coeff -> gl_COEFF
+   Order -> gl_ORDER
+   Domain -> gl_DOMAIN
 
 --------------------------------------------------------------------------------
 
@@ -379,26 +353,17 @@ mapGrid2 =
 evalMesh1 :: PolygonMode -> (GLint, GLint) -> IO ()
 evalMesh1 m (p1, p2) = glEvalMesh1 (marshalPolygonMode m) p1 p2
 
-foreign import CALLCONV unsafe "glEvalMesh1" glEvalMesh1 ::
-   GLenum -> GLint -> GLint -> IO ()
-
 evalMesh2 :: PolygonMode -> (GLint, GLint) -> (GLint, GLint) -> IO ()
 evalMesh2 m (p1, p2) (q1, q2) = glEvalMesh2 (marshalPolygonMode m) p1 p2 q1 q2
 
-foreign import CALLCONV unsafe "glEvalMesh2" glEvalMesh2 ::
-   GLenum -> GLint -> GLint -> GLint -> GLint -> IO ()
-
 --------------------------------------------------------------------------------
 
-foreign import CALLCONV unsafe "glEvalPoint1" evalPoint1 ::
-   GLint -> IO ()
+evalPoint1 :: GLint -> IO ()
+evalPoint1 = glEvalPoint1
 
 evalPoint2 :: (GLint, GLint) -> IO ()
 evalPoint2 = uncurry glEvalPoint2
 
-foreign import CALLCONV unsafe "glEvalPoint2" glEvalPoint2 ::
-   GLint -> GLint -> IO ()
-
 --------------------------------------------------------------------------------
 
 autoNormal :: StateVar Capability
