@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.Feedback
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -17,20 +17,21 @@ module Graphics.Rendering.OpenGL.GL.Feedback (
    getFeedbackTokens, PassThroughValue(..), passThrough
 ) where
 
-import Control.Monad ( liftM2, liftM3, liftM4 )
-import Foreign.Marshal.Array ( allocaArray )
-import Foreign.Ptr ( Ptr, plusPtr )
-import Foreign.Storable ( Storable(sizeOf) )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLenum, GLint, GLsizei, GLfloat )
-import Graphics.Rendering.OpenGL.GL.IOState (
-   IOState, getIOState, peekIOState, evalIOState, nTimes )
-import Graphics.Rendering.OpenGL.GL.RenderMode ( withRenderMode )
-import Graphics.Rendering.OpenGL.GL.Selection ( RenderMode(Feedback) )
-import Graphics.Rendering.OpenGL.GL.StateVar ( HasGetter(get) )
-import Graphics.Rendering.OpenGL.GL.VertexSpec (
-   Vertex2(..), Vertex3(..), Vertex4(..), Index1(..), Color4(..),
-   TexCoord4(..), rgbaMode )
+import Control.Monad
+import Data.StateVar
+import Data.Tensor
+import Foreign.Marshal.Array
+import Foreign.Ptr
+import Foreign.Storable
+import Graphics.Rendering.OpenGL.GL.IOState
+import Graphics.Rendering.OpenGL.GL.RenderMode
+import Graphics.Rendering.OpenGL.GL.VertexSpec
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glFeedbackBuffer, glPassThrough, gl_2D, gl_3D, gl_3D_COLOR,
+   gl_3D_COLOR_TEXTURE, gl_4D_COLOR_TEXTURE, gl_BITMAP_TOKEN,
+   gl_COPY_PIXEL_TOKEN, gl_DRAW_PIXEL_TOKEN, gl_LINE_RESET_TOKEN, gl_LINE_TOKEN,
+   gl_PASS_THROUGH_TOKEN, gl_POINT_TOKEN, gl_POLYGON_TOKEN )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -69,14 +70,14 @@ data FeedbackTag =
 
 unmarshalFeedbackTag :: GLenum -> FeedbackTag
 unmarshalFeedbackTag x
-   | x == 0x701 = PointTag
-   | x == 0x702 = LineTag
-   | x == 0x707 = LineResetTag
-   | x == 0x703 = PolygonTag
-   | x == 0x704 = BitmapTag
-   | x == 0x705 = DrawPixelTag
-   | x == 0x706 = CopyPixelTag
-   | x == 0x700 = PassThroughTag
+   | x == gl_POINT_TOKEN = PointTag
+   | x == gl_LINE_TOKEN = LineTag
+   | x == gl_LINE_RESET_TOKEN = LineResetTag
+   | x == gl_POLYGON_TOKEN = PolygonTag
+   | x == gl_BITMAP_TOKEN = BitmapTag
+   | x == gl_DRAW_PIXEL_TOKEN = DrawPixelTag
+   | x == gl_COPY_PIXEL_TOKEN = CopyPixelTag
+   | x == gl_PASS_THROUGH_TOKEN = PassThroughTag
    | otherwise = error ("unmarshalFeedbackTag: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
@@ -91,11 +92,11 @@ data FeedbackType =
 
 marshalFeedbackType :: FeedbackType -> GLenum
 marshalFeedbackType x = case x of
-   TwoD -> 0x600
-   ThreeD -> 0x601
-   ThreeDColor -> 0x602
-   ThreeDColorTexture -> 0x603
-   FourDColorTexture -> 0x604
+   TwoD -> gl_2D
+   ThreeD -> gl_3D
+   ThreeDColor -> gl_3D_COLOR
+   ThreeDColorTexture -> gl_3D_COLOR_TEXTURE
+   FourDColorTexture -> gl_4D_COLOR_TEXTURE
 
 --------------------------------------------------------------------------------
 
@@ -108,9 +109,6 @@ getFeedbackTokens bufSize feedbackType action =
       tokens <- parseFeedbackBuffer numValues buf feedbackType
       return (value, tokens)
 
-foreign import CALLCONV unsafe "glFeedbackBuffer" glFeedbackBuffer ::
-   GLsizei -> GLenum -> Ptr GLfloat -> IO ()
-
 --------------------------------------------------------------------------------
 
 parseFeedbackBuffer ::
@@ -199,5 +197,5 @@ parseGLfloat = peekIOState
 newtype PassThroughValue = PassThroughValue GLfloat
    deriving ( Eq, Ord, Show )
 
-foreign import CALLCONV unsafe "glPassThrough" passThrough ::
-   PassThroughValue -> IO ()
+passThrough :: PassThroughValue -> IO ()
+passThrough (PassThroughValue ptv) = glPassThrough ptv
