@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.Framebuffer
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -19,10 +19,10 @@ module Graphics.Rendering.OpenGL.GL.Framebuffer (
    rgbaBits, stencilBits, depthBits, accumBits, rgbaSignedComponents,
 
    -- * Selecting a Buffer for Writing
-   BufferMode(..), drawBuffer, drawBuffers, maxDrawBuffers,
+   DrawBufferIndex, BufferMode(..), drawBuffer, drawBuffers, drawBufferi, maxDrawBuffers,
 
    -- * Fine Control of Buffer Updates
-   indexMask, colorMask, stencilMask, stencilMaskSeparate, depthMask,
+   indexMask, colorMask, colorMaski, stencilMask, stencilMaskSeparate, depthMask,
 
    -- * Clearing the Buffers
    ClearBuffer(..), clear,
@@ -32,50 +32,22 @@ module Graphics.Rendering.OpenGL.GL.Framebuffer (
    AccumOp(..), accum
 ) where
 
-import Control.Monad ( liftM4 )
-import Data.List ( genericLength )
-import Data.Maybe ( isJust, fromJust )
-import Foreign.Marshal.Array ( withArray )
-import Foreign.Ptr ( Ptr )
-import Graphics.Rendering.OpenGL.GL.BufferMode (
-   BufferMode(..), marshalBufferMode, unmarshalBufferMode )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   marshalCapability, unmarshalCapability )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLboolean, GLenum, GLsizei, GLint, GLuint, GLbitfield, GLfloat, GLclampf,
-   GLclampd,
-#ifdef __HADDOCK__
-   Capability(..)
-#else
-   Capability
-#endif
-   )
-import Graphics.Rendering.OpenGL.GL.Extensions (
-   FunPtr, unsafePerformIO, Invoker, getProcAddress )
-import Graphics.Rendering.OpenGL.GL.Face ( Face, marshalFace )
-import Graphics.Rendering.OpenGL.GL.GLboolean ( unmarshalGLboolean )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetAuxBuffers,GetDoublebuffer,GetStereo,GetRedBits,GetGreenBits,
-            GetBlueBits,GetAlphaBits,GetStencilBits,GetDepthBits,
-            GetAccumRedBits,GetAccumGreenBits,GetAccumBlueBits,
-            GetAccumAlphaBits,GetRGBASignedComponents,GetDrawBuffer,
-            GetIndexWritemask,GetColorWritemask,GetDepthWritemask,
-            GetStencilWritemask,GetColorClearValue,GetIndexClearValue,
-            GetDepthClearValue,GetStencilClearValue,GetAccumClearValue,
-            GetMaxDrawBuffers,GetDrawBufferN),
-   getInteger1, getInteger4, getBoolean1, getBoolean4, getEnum1, getSizei1,
-   getFloat1, getFloat4, getDouble1 )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   HasGetter(get), GettableStateVar, makeGettableStateVar,
-   SettableStateVar, makeSettableStateVar,
-   StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.VertexSpec (
-   Color4(Color4), Index1(Index1) )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal ( recordInvalidValue )
-
---------------------------------------------------------------------------------
-
-#include "HsOpenGLExt.h"
+import Control.Monad
+import Data.List
+import Data.Maybe
+import Data.StateVar
+import Foreign.Marshal.Array
+import Graphics.Rendering.OpenGL.GL.BufferMode
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.Face
+import Graphics.Rendering.OpenGL.GL.GLboolean
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.VertexSpec
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glAccum, glClearAccum, glClearIndex, glIndexMask, gl_ACCUM,
+   gl_ACCUM_BUFFER_BIT, gl_ADD, gl_LOAD, gl_MULT, gl_RETURN )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -131,7 +103,7 @@ rgbaSignedComponents =
 
 -- | When colors are written to the framebuffer, they are written into the color
 -- buffers specified by 'drawBuffer'.
--- 
+--
 -- If more than one color buffer is selected for drawing, then blending or
 -- logical operations are computed and applied independently for each color
 -- buffer and can produce different results in each buffer.
@@ -150,11 +122,10 @@ drawBuffer =
       (getEnum1 unmarshalBufferMode GetDrawBuffer)
       (maybe recordInvalidValue glDrawBuffer . marshalBufferMode)
 
-foreign import CALLCONV unsafe "glDrawBuffer" glDrawBuffer ::
-   GLenum -> IO ()
-
 --------------------------------------------------------------------------------
 
+type DrawBufferIndex = GLuint
+
 -- | 'drawBuffers' defines the draw buffers to which all fragment colors are
 -- written. The draw buffers being defined correspond in order to the respective
 -- fragment colors. The draw buffer for fragment colors beyond those specified
@@ -192,7 +163,13 @@ setDrawBuffers modes = do
               glDrawBuffers (genericLength ms)
       else recordInvalidValue
 
-EXTENSION_ENTRY("GL_ARB_draw_buffers or OpenGL 2.0",glDrawBuffers,GLsizei -> Ptr GLenum -> IO ())
+-- | 'drawBufferi' is a fast query function. For indices in the range 0..maxDrawBuffers it's results
+-- are the same as selection the index from the list returned by drawBuffers. Though this function
+-- only uses one gl-function call instead of maxDrawBuffers + 1.
+drawBufferi :: DrawBufferIndex -> GettableStateVar BufferMode
+drawBufferi ind = makeGettableStateVar
+    (getEnum1 unmarshalBufferMode . GetDrawBufferN $ fromIntegral ind)
+
 
 --------------------------------------------------------------------------------
 
@@ -221,8 +198,6 @@ indexMask :: StateVar GLuint
 indexMask =
    makeStateVar (getInteger1 fromIntegral GetIndexWritemask) glIndexMask
 
-foreign import CALLCONV unsafe "glIndexMask" glIndexMask :: GLuint -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Controls whether the individual color components in the framebuffer can or
@@ -247,9 +222,18 @@ colorMask =
                                         (marshalCapability g)
                                         (marshalCapability b)
                                         (marshalCapability a))
-
-foreign import CALLCONV unsafe "glColorMask" glColorMask ::
-   GLboolean -> GLboolean -> GLboolean -> GLboolean -> IO ()
+-- | 'colorMaski' is a version of 'colorMask' that only applies to the specified drawbuffer
+colorMaski :: DrawBufferIndex -> StateVar (Color4 Capability)
+colorMaski x = makeStateVar
+      (getBoolean4i (\r g b a -> Color4 (unmarshalCapability r)
+                                       (unmarshalCapability g)
+                                       (unmarshalCapability b)
+                                       (unmarshalCapability a))
+                    GetColorWritemask x)
+      (\(Color4 r g b a) -> glColorMaski (x) (marshalCapability r)
+                                        (marshalCapability g)
+                                        (marshalCapability b)
+                                        (marshalCapability a))
 
 --------------------------------------------------------------------------------
 
@@ -260,8 +244,6 @@ depthMask :: StateVar Capability
 depthMask = makeStateVar (getBoolean1 unmarshalCapability GetDepthWritemask)
                          (glDepthMask . marshalCapability)
 
-foreign import CALLCONV unsafe "glDepthMask" glDepthMask :: GLboolean -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Controls the writing of individual bits in the stencil planes. The least
@@ -275,15 +257,11 @@ stencilMask :: StateVar GLuint
 stencilMask =
    makeStateVar (getInteger1 fromIntegral GetStencilWritemask) glStencilMask
 
-foreign import CALLCONV unsafe "glStencilMask" glStencilMask :: GLuint -> IO ()
-
 stencilMaskSeparate :: Face -> SettableStateVar GLuint
 stencilMaskSeparate face =
    makeSettableStateVar $
       glStencilMaskSeparate (marshalFace face)
 
-EXTENSION_ENTRY("OpenGL 2.0",glStencilMaskSeparate,GLenum -> GLuint -> IO ())
-
 --------------------------------------------------------------------------------
 
 -- | The buffers which can be cleared with 'clear'.
@@ -296,11 +274,11 @@ data ClearBuffer =
    deriving ( Eq, Ord, Show )
 
 marshalClearBuffer :: ClearBuffer -> GLbitfield
-marshalClearBuffer x = case x of
-   ColorBuffer -> 0x4000
-   AccumBuffer -> 0x200
-   StencilBuffer -> 0x400
-   DepthBuffer -> 0x100
+marshalClearBuffer x = fromIntegral $ case x of
+   ColorBuffer -> gl_COLOR_BUFFER_BIT
+   AccumBuffer -> gl_ACCUM_BUFFER_BIT
+   StencilBuffer -> gl_STENCIL_BUFFER_BIT
+   DepthBuffer -> gl_DEPTH_BUFFER_BIT
 
 --------------------------------------------------------------------------------
 
@@ -324,8 +302,6 @@ marshalClearBuffer x = case x of
 clear :: [ClearBuffer] -> IO ()
 clear = glClear . sum . map marshalClearBuffer
 
-foreign import CALLCONV unsafe "glClear" glClear :: GLbitfield -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Controls the red, green, blue, and alpha values used by 'clear' to clear
@@ -333,12 +309,9 @@ foreign import CALLCONV unsafe "glClear" glClear :: GLbitfield -> IO ()
 -- [0, 1]. Initially, all values are 0.
 
 clearColor :: StateVar (Color4 GLclampf)
-clearColor = makeStateVar (getFloat4 Color4 GetColorClearValue)
+clearColor = makeStateVar (getClampf4 Color4 GetColorClearValue)
                           (\(Color4 r g b a) -> glClearColor r g b a)
 
-foreign import CALLCONV unsafe "glClearColor" glClearColor ::
-    GLclampf -> GLclampf -> GLclampf -> GLclampf -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Controls the index /c/ used by 'clear' to clear the color index buffers.
@@ -351,8 +324,6 @@ clearIndex :: StateVar (Index1 GLfloat)
 clearIndex = makeStateVar (getFloat1 Index1 GetIndexClearValue)
                           (\(Index1 i) -> glClearIndex i)
 
-foreign import CALLCONV unsafe "glClearIndex" glClearIndex :: GLfloat -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Controls the depth value used by 'clear' to clear the depth buffer. Values
@@ -360,9 +331,7 @@ foreign import CALLCONV unsafe "glClearIndex" glClearIndex :: GLfloat -> IO ()
 -- is 1.
 
 clearDepth :: StateVar GLclampd
-clearDepth = makeStateVar (getDouble1 id GetDepthClearValue) glClearDepth
-
-foreign import CALLCONV unsafe "glClearDepth" glClearDepth :: GLclampd -> IO ()
+clearDepth = makeStateVar (getClampd1 id GetDepthClearValue) glClearDepth
 
 --------------------------------------------------------------------------------
 
@@ -373,8 +342,6 @@ foreign import CALLCONV unsafe "glClearDepth" glClearDepth :: GLclampd -> IO ()
 clearStencil :: StateVar GLint
 clearStencil = makeStateVar (getInteger1 id GetStencilClearValue) glClearStencil
 
-foreign import CALLCONV unsafe "glClearStencil" glClearStencil :: GLint -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Controls the red, green, blue, and alpha values used by 'clear' to clear
@@ -386,9 +353,6 @@ clearAccum =
    makeStateVar (getFloat4 Color4 GetAccumClearValue)
                 (\(Color4 r g b a) -> glClearAccum r g b a)
 
-foreign import CALLCONV unsafe "glClearAccum" glClearAccum ::
-   GLfloat -> GLfloat -> GLfloat -> GLfloat -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | An operation on the accumulation buffer.
@@ -430,11 +394,11 @@ data AccumOp =
 
 marshalAccumOp :: AccumOp -> GLenum
 marshalAccumOp x = case x of
-   Accum -> 0x100
-   Load -> 0x101
-   Return -> 0x102
-   Mult -> 0x103
-   Add -> 0x104
+   Accum -> gl_ACCUM
+   Load -> gl_LOAD
+   Return -> gl_RETURN
+   Mult -> gl_MULT
+   Add -> gl_ADD
 
 --------------------------------------------------------------------------------
 
@@ -466,5 +430,3 @@ marshalAccumOp x = case x of
 
 accum :: AccumOp -> GLfloat -> IO ()
 accum = glAccum . marshalAccumOp
-
-foreign import CALLCONV unsafe "glAccum" glAccum :: GLenum -> GLfloat -> IO ()
