@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.Hints
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -16,13 +16,14 @@ module Graphics.Rendering.OpenGL.GL.Hints (
    HintTarget(..), HintMode(..), hint
 ) where
 
-import Graphics.Rendering.OpenGL.GL.BasicTypes ( GLenum )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetPerspectiveCorrectionHint,GetPointSmoothHint,GetLineSmoothHint,
-            GetPolygonSmoothHint,GetFogHint,GetGenerateMipmapHint,
-            GetTextureCompressionHint,GetPackCMYKHint,GetUnpackCMYKHint),
-   getEnum1 )
-import Graphics.Rendering.OpenGL.GL.StateVar ( StateVar, makeStateVar )
+import Data.StateVar
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   gl_FOG_HINT, gl_GENERATE_MIPMAP_HINT, gl_PERSPECTIVE_CORRECTION_HINT,
+   gl_POINT_SMOOTH_HINT )
+import Graphics.Rendering.OpenGL.Raw.Core31
+import Graphics.Rendering.OpenGL.Raw.EXT.Cmyka (
+   gl_PACK_CMYK_HINT, gl_UNPACK_CMYK_HINT )
 
 --------------------------------------------------------------------------------
 
@@ -40,17 +41,17 @@ data HintTarget =
 
 marshalHintTarget :: HintTarget -> GLenum
 marshalHintTarget x = case x of
-   PerspectiveCorrection -> 0xc50
-   PointSmooth -> 0xc51
-   LineSmooth -> 0xc52
-   PolygonSmooth -> 0xc53
-   Fog -> 0xc54
-   GenerateMipmap -> 0x8192
-   TextureCompression -> 0x84ef
-   PackCMYK -> 0x800e
-   UnpackCMYK -> 0x800f
+   PerspectiveCorrection -> gl_PERSPECTIVE_CORRECTION_HINT
+   PointSmooth -> gl_POINT_SMOOTH_HINT
+   LineSmooth -> gl_LINE_SMOOTH_HINT
+   PolygonSmooth -> gl_POLYGON_SMOOTH_HINT
+   Fog -> gl_FOG_HINT
+   GenerateMipmap -> gl_GENERATE_MIPMAP_HINT
+   TextureCompression -> gl_TEXTURE_COMPRESSION_HINT
+   PackCMYK -> gl_PACK_CMYK_HINT
+   UnpackCMYK -> gl_UNPACK_CMYK_HINT
 
-hintTargetToGetPName :: HintTarget -> GetPName
+hintTargetToGetPName :: HintTarget -> PName1I
 hintTargetToGetPName x = case x of
    PerspectiveCorrection -> GetPerspectiveCorrectionHint
    PointSmooth -> GetPointSmoothHint
@@ -72,15 +73,15 @@ data HintMode =
 
 marshalHintMode :: HintMode -> GLenum
 marshalHintMode x = case x of
-   DontCare -> 0x1100
-   Fastest -> 0x1101
-   Nicest -> 0x1102
+   DontCare -> gl_DONT_CARE
+   Fastest -> gl_FASTEST
+   Nicest -> gl_NICEST
 
 unmarshalHintMode :: GLenum -> HintMode
 unmarshalHintMode x
-   | x == 0x1100 = DontCare
-   | x == 0x1101 = Fastest
-   | x == 0x1102 = Nicest
+   | x == gl_DONT_CARE = DontCare
+   | x == gl_FASTEST = Fastest
+   | x == gl_NICEST = Nicest
    | otherwise = error ("unmarshalHintMode: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
@@ -90,5 +91,3 @@ hint t =
    makeStateVar
       (getEnum1 unmarshalHintMode (hintTargetToGetPName t))
       (glHint (marshalHintTarget t) . marshalHintMode)
-
-foreign import CALLCONV unsafe "glHint" glHint :: GLenum -> GLenum -> IO ()
