@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.PixelRectangles.ColorTable
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -21,35 +21,32 @@ module Graphics.Rendering.OpenGL.GL.PixelRectangles.ColorTable (
    colorTableRGBASizes, colorTableLuminanceSize, colorTableIntesitySize,
 ) where
 
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Marshal.Utils ( with )
-import Foreign.Ptr ( Ptr )
-import Foreign.Storable ( Storable(..) )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   EnableCap(CapColorTable,CapPostConvolutionColorTable,
-             CapPostColorMatrixColorTable,CapTextureColorTable),
-   makeCapability )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLenum, GLint, GLsizei, GLfloat, Capability )
-import Graphics.Rendering.OpenGL.GL.CoordTrans ( Position(..) )
-import Graphics.Rendering.OpenGL.GL.Extensions (
-   FunPtr, unsafePerformIO, Invoker, getProcAddress )
-import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek1 )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.Rasterization (
-    PixelData(..) )
-import Graphics.Rendering.OpenGL.GL.PixelData ( withPixelData )
-import Graphics.Rendering.OpenGL.GL.Texturing.PixelInternalFormat (
-   PixelInternalFormat(..), marshalPixelInternalFormat',
-   unmarshalPixelInternalFormat )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar, StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.VertexSpec ( Color4(..) )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal (
-   recordInvalidEnum )
-
---------------------------------------------------------------------------------
-
-#include "HsOpenGLExt.h"
+import Data.StateVar
+import Foreign.Marshal.Alloc
+import Foreign.Marshal.Utils
+import Foreign.Ptr
+import Foreign.Storable
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.CoordTrans
+import Graphics.Rendering.OpenGL.GL.PeekPoke
+import Graphics.Rendering.OpenGL.GL.PixelData
+import Graphics.Rendering.OpenGL.GL.Texturing.PixelInternalFormat
+import Graphics.Rendering.OpenGL.GL.VertexSpec
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glColorSubTable, glColorTable, glColorTableParameterfv, glCopyColorSubTable,
+   glCopyColorTable, glGetColorTable, glGetColorTableParameterfv,
+   glGetColorTableParameteriv, gl_COLOR_TABLE, gl_COLOR_TABLE_ALPHA_SIZE,
+   gl_COLOR_TABLE_BIAS, gl_COLOR_TABLE_BLUE_SIZE, gl_COLOR_TABLE_FORMAT,
+   gl_COLOR_TABLE_GREEN_SIZE, gl_COLOR_TABLE_INTENSITY_SIZE,
+   gl_COLOR_TABLE_LUMINANCE_SIZE, gl_COLOR_TABLE_RED_SIZE, gl_COLOR_TABLE_SCALE,
+   gl_COLOR_TABLE_WIDTH, gl_POST_COLOR_MATRIX_COLOR_TABLE,
+   gl_POST_CONVOLUTION_COLOR_TABLE, gl_PROXY_COLOR_TABLE,
+   gl_PROXY_POST_COLOR_MATRIX_COLOR_TABLE,
+   gl_PROXY_POST_CONVOLUTION_COLOR_TABLE )
+import Graphics.Rendering.OpenGL.Raw.Core31
+import Graphics.Rendering.OpenGL.Raw.EXT.SharedTexturePalette (
+   gl_SHARED_TEXTURE_PALETTE )
 
 --------------------------------------------------------------------------------
 
@@ -95,15 +92,16 @@ data ColorTable =
 
 marshalColorTable :: ColorTable -> GLenum
 marshalColorTable x = case x of
-   ColorTable -> 0x80d0
-   PostConvolutionColorTable -> 0x80d1
-   PostColorMatrixColorTable -> 0x80d2
-   Texture1DColorTable -> 0xde0
-   Texture2DColorTable -> 0xde1
-   Texture3DColorTable -> 0x806f
-   TextureCubeMapColorTable -> 0x8513
+   ColorTable -> gl_COLOR_TABLE
+   PostConvolutionColorTable -> gl_POST_CONVOLUTION_COLOR_TABLE
+   PostColorMatrixColorTable -> gl_POST_COLOR_MATRIX_COLOR_TABLE
+   Texture1DColorTable -> gl_TEXTURE_1D
+   Texture2DColorTable -> gl_TEXTURE_2D
+   Texture3DColorTable -> gl_TEXTURE_3D
+   TextureCubeMapColorTable -> gl_TEXTURE_CUBE_MAP
+   -- TODO: Use TEXTURE_COLOR_TABLE_SGI from SGI_texture_color_table extension
    TextureColorTable -> 0x80bc
-   SharedTexturePalette -> 0x81fb
+   SharedTexturePalette -> gl_SHARED_TEXTURE_PALETTE
 
 --------------------------------------------------------------------------------
 
@@ -117,13 +115,14 @@ data Proxy =
 marshalProxyColorTable :: Proxy -> ColorTable -> Maybe GLenum
 marshalProxyColorTable NoProxy x = Just (marshalColorTable x)
 marshalProxyColorTable Proxy   x = case x of
-   ColorTable -> Just 0x80d3
-   PostConvolutionColorTable -> Just 0x80d4
-   PostColorMatrixColorTable -> Just 0x80d5
-   Texture1DColorTable -> Just 0x8063
-   Texture2DColorTable -> Just 0x8064
-   Texture3DColorTable -> Just 0x8070
-   TextureCubeMapColorTable -> Just 0x851b
+   ColorTable -> Just gl_PROXY_COLOR_TABLE
+   PostConvolutionColorTable -> Just gl_PROXY_POST_CONVOLUTION_COLOR_TABLE
+   PostColorMatrixColorTable -> Just gl_PROXY_POST_COLOR_MATRIX_COLOR_TABLE
+   Texture1DColorTable -> Just gl_PROXY_TEXTURE_1D
+   Texture2DColorTable -> Just gl_PROXY_TEXTURE_2D
+   Texture3DColorTable -> Just gl_PROXY_TEXTURE_3D
+   TextureCubeMapColorTable -> Just gl_PROXY_TEXTURE_CUBE_MAP
+   -- TODO: Use PROXY_TEXTURE_COLOR_TABLE_SGI from SGI_texture_color_table extension
    TextureColorTable -> Just 0x80bd
    SharedTexturePalette -> Nothing
 
@@ -137,40 +136,30 @@ colorTable proxy ct int w pd =
             glColorTable target (marshalPixelInternalFormat' int) w)
          (marshalProxyColorTable proxy ct)
 
-EXTENSION_ENTRY("GL_ARB_imaging",glColorTable,GLenum -> GLenum -> GLsizei -> GLenum -> GLenum -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 getColorTable :: ColorTable -> PixelData a -> IO ()
 getColorTable ct pd =
    withPixelData pd $ glGetColorTable (marshalColorTable ct)
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetColorTable,GLenum -> GLenum -> GLenum -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 copyColorTable :: ColorTable -> PixelInternalFormat -> Position -> GLsizei -> IO ()
 copyColorTable ct int (Position x y) =
    glCopyColorTable (marshalColorTable ct) (marshalPixelInternalFormat' int) x y
 
-EXTENSION_ENTRY("GL_ARB_imaging",glCopyColorTable,GLenum -> GLenum -> GLint -> GLint -> GLsizei -> IO ())
-
 --------------------------------------------------------------------------------
 
 colorSubTable :: ColorTable -> GLsizei -> GLsizei -> PixelData a -> IO ()
 colorSubTable ct start count pd =
    withPixelData pd $ glColorSubTable (marshalColorTable ct) start count
 
-EXTENSION_ENTRY("GL_ARB_imaging",glColorSubTable,GLenum -> GLsizei -> GLsizei -> GLenum -> GLenum -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 copyColorSubTable :: ColorTable -> GLsizei -> Position -> GLsizei -> IO ()
 copyColorSubTable ct start (Position x y) =
    glCopyColorSubTable (marshalColorTable ct) start x y
 
-EXTENSION_ENTRY("GL_ARB_imaging",glCopyColorSubTable,GLenum -> GLsizei -> GLint -> GLint -> GLsizei -> IO ())
-
 --------------------------------------------------------------------------------
 
 data ColorTablePName =
@@ -187,16 +176,16 @@ data ColorTablePName =
 
 marshalColorTablePName :: ColorTablePName -> GLenum
 marshalColorTablePName x = case x of
-   ColorTableScale -> 0x80d6
-   ColorTableBias -> 0x80d7
-   ColorTableFormat -> 0x80d8
-   ColorTableWidth -> 0x80d9
-   ColorTableRedSize -> 0x80da
-   ColorTableGreenSize -> 0x80db
-   ColorTableBlueSize -> 0x80dc
-   ColorTableAlphaSize -> 0x80dd
-   ColorTableLuminanceSize -> 0x80de
-   ColorTableIntensitySize -> 0x80df
+   ColorTableScale -> gl_COLOR_TABLE_SCALE
+   ColorTableBias -> gl_COLOR_TABLE_BIAS
+   ColorTableFormat -> gl_COLOR_TABLE_FORMAT
+   ColorTableWidth -> gl_COLOR_TABLE_WIDTH
+   ColorTableRedSize -> gl_COLOR_TABLE_RED_SIZE
+   ColorTableGreenSize -> gl_COLOR_TABLE_GREEN_SIZE
+   ColorTableBlueSize -> gl_COLOR_TABLE_BLUE_SIZE
+   ColorTableAlphaSize -> gl_COLOR_TABLE_ALPHA_SIZE
+   ColorTableLuminanceSize -> gl_COLOR_TABLE_LUMINANCE_SIZE
+   ColorTableIntensitySize -> gl_COLOR_TABLE_INTENSITY_SIZE
 
 --------------------------------------------------------------------------------
 
@@ -213,24 +202,20 @@ colorTableScaleBias p s =
    where ct = colorTableStageToColorTable s
 
 getColorTableParameterC4f ::
-   ColorTable -> ColorTablePName -> IO (Color4 GLfloat) 
+   ColorTable -> ColorTablePName -> IO (Color4 GLfloat)
 getColorTableParameterC4f ct p =
    alloca $ \buf -> do
       glGetColorTableParameterfv
          (marshalColorTable ct)
          (marshalColorTablePName p)
-         buf
+         (castPtr buf)
       peek buf
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetColorTableParameterfv,GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ())
-
 colorTableParameterC4f ::
    ColorTable -> ColorTablePName -> Color4 GLfloat -> IO ()
 colorTableParameterC4f ct p c =
-   with c $
-      glColorTableParameterfv (marshalColorTable ct) (marshalColorTablePName p)
-
-EXTENSION_ENTRY("GL_ARB_imaging",glColorTableParameterfv,GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ())
+   with c $ \ptr ->
+      glColorTableParameterfv (marshalColorTable ct) (marshalColorTablePName p) (castPtr ptr)
 
 --------------------------------------------------------------------------------
 
@@ -248,8 +233,6 @@ getColorTableParameteri f ct p =
          buf
       peek1 f buf
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetColorTableParameteriv,GLenum -> GLenum -> Ptr GLint -> IO ())
-
 --------------------------------------------------------------------------------
 
 colorTableWidth :: ColorTable -> GettableStateVar GLsizei
