@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.PixelRectangles.Convolution
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -25,35 +25,30 @@ module Graphics.Rendering.OpenGL.GL.PixelRectangles.Convolution (
    convolutionFilterScale, convolutionFilterBias,
 ) where
 
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Marshal.Utils ( with )
-import Foreign.Ptr ( Ptr, nullPtr )
-import Foreign.Storable ( Storable(..) )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   EnableCap(CapConvolution1D, CapConvolution2D,CapSeparable2D),
-   makeCapability )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLenum, GLint, GLsizei, GLfloat, Capability )
-import Graphics.Rendering.OpenGL.GL.CoordTrans ( Position(..), Size(..) )
-import Graphics.Rendering.OpenGL.GL.Extensions (
-   FunPtr, unsafePerformIO, Invoker, getProcAddress )
-import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek1 )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.ColorTable (
-   PixelInternalFormat )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.Rasterization (
-    PixelData(..) )
-import Graphics.Rendering.OpenGL.GL.PixelData ( withPixelData )
-import Graphics.Rendering.OpenGL.GL.Texturing.PixelInternalFormat (
-   marshalPixelInternalFormat' )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar, StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.VertexSpec ( Color4(..) )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal (
-   recordInvalidValue )
-
---------------------------------------------------------------------------------
-
-#include "HsOpenGLExt.h"
+import Data.StateVar
+import Foreign.Marshal.Alloc
+import Foreign.Marshal.Utils
+import Foreign.Ptr
+import Foreign.Storable
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.CoordTrans
+import Graphics.Rendering.OpenGL.GL.PeekPoke
+import Graphics.Rendering.OpenGL.GL.PixelData
+import Graphics.Rendering.OpenGL.GL.Texturing.PixelInternalFormat
+import Graphics.Rendering.OpenGL.GL.VertexSpec
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glConvolutionFilter1D, glConvolutionFilter2D, glConvolutionParameterfv,
+   glConvolutionParameteri, glCopyConvolutionFilter1D,
+   glCopyConvolutionFilter2D, glGetConvolutionFilter,
+   glGetConvolutionParameterfv, glGetConvolutionParameteriv,
+   glGetSeparableFilter, glSeparableFilter2D, gl_CONSTANT_BORDER,
+   gl_CONVOLUTION_1D, gl_CONVOLUTION_2D, gl_CONVOLUTION_BORDER_COLOR,
+   gl_CONVOLUTION_BORDER_MODE, gl_CONVOLUTION_FILTER_BIAS,
+   gl_CONVOLUTION_FILTER_SCALE, gl_CONVOLUTION_FORMAT, gl_CONVOLUTION_HEIGHT,
+   gl_CONVOLUTION_WIDTH, gl_MAX_CONVOLUTION_HEIGHT, gl_MAX_CONVOLUTION_WIDTH,
+   gl_REDUCE, gl_REPLICATE_BORDER, gl_SEPARABLE_2D )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -65,9 +60,9 @@ data ConvolutionTarget =
 
 marshalConvolutionTarget :: ConvolutionTarget -> GLenum
 marshalConvolutionTarget x = case x of
-   Convolution1D -> 0x8010
-   Convolution2D -> 0x8011
-   Separable2D -> 0x8012
+   Convolution1D -> gl_CONVOLUTION_1D
+   Convolution2D -> gl_CONVOLUTION_2D
+   Separable2D -> gl_SEPARABLE_2D
 
 convolutionTargetToEnableCap :: ConvolutionTarget -> EnableCap
 convolutionTargetToEnableCap x = case x of
@@ -89,8 +84,6 @@ convolutionFilter1D int w pd =
          (marshalConvolutionTarget Convolution1D)
          (marshalPixelInternalFormat' int) w
 
-EXTENSION_ENTRY("GL_ARB_imaging",glConvolutionFilter1D,GLenum -> GLenum -> GLsizei -> GLenum -> GLenum -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 getConvolutionFilter1D :: PixelData a -> IO ()
@@ -100,8 +93,6 @@ getConvolutionFilter :: ConvolutionTarget -> PixelData a -> IO ()
 getConvolutionFilter t pd =
    withPixelData pd $ glGetConvolutionFilter (marshalConvolutionTarget t)
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetConvolutionFilter,GLenum -> GLenum -> GLenum -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 convolutionFilter2D :: PixelInternalFormat -> Size -> PixelData a -> IO ()
@@ -111,8 +102,6 @@ convolutionFilter2D int (Size w h) pd =
          (marshalConvolutionTarget Convolution2D)
          (marshalPixelInternalFormat' int) w h
 
-EXTENSION_ENTRY("GL_ARB_imaging",glConvolutionFilter2D,GLenum -> GLenum -> GLsizei -> GLsizei -> GLenum -> GLenum -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 getConvolutionFilter2D :: PixelData a -> IO ()
@@ -131,8 +120,6 @@ separableFilter2D int (Size w h) pdRow pdCol =
                     (marshalPixelInternalFormat' int) w h f1 d1 p1 p2
             else recordInvalidValue
 
-EXTENSION_ENTRY("GL_ARB_imaging",glSeparableFilter2D,GLenum -> GLenum -> GLsizei -> GLsizei -> GLenum -> GLenum -> Ptr a -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 getSeparableFilter2D :: PixelData a -> PixelData a -> IO ()
@@ -144,8 +131,6 @@ getSeparableFilter2D pdRow pdCol =
                     (marshalConvolutionTarget Separable2D) f1 d1 p1 p2 nullPtr
             else recordInvalidValue
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetSeparableFilter,GLenum -> GLenum -> GLenum -> Ptr a -> Ptr a -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 copyConvolutionFilter1D :: PixelInternalFormat -> Position -> GLsizei -> IO ()
@@ -154,8 +139,6 @@ copyConvolutionFilter1D int (Position x y) =
       (marshalConvolutionTarget Convolution1D) (marshalPixelInternalFormat' int)
       x y
 
-EXTENSION_ENTRY("GL_ARB_imaging",glCopyConvolutionFilter1D,GLenum -> GLenum -> GLint -> GLint -> GLsizei -> IO ())
-
 --------------------------------------------------------------------------------
 
 copyConvolutionFilter2D :: PixelInternalFormat -> Position -> Size -> IO ()
@@ -164,8 +147,6 @@ copyConvolutionFilter2D int (Position x y) (Size w h) =
       (marshalConvolutionTarget Convolution2D) (marshalPixelInternalFormat' int)
       x y w h
 
-EXTENSION_ENTRY("GL_ARB_imaging",glCopyConvolutionFilter2D,GLenum -> GLenum -> GLint -> GLint -> GLsizei -> GLsizei -> IO ())
-
 --------------------------------------------------------------------------------
 
 data ConvolutionParameter =
@@ -182,15 +163,15 @@ data ConvolutionParameter =
 
 marshalConvolutionParameter :: ConvolutionParameter -> GLenum
 marshalConvolutionParameter x = case x of
-   ConvolutionBorderColor -> 0x8154
-   ConvolutionBorderMode -> 0x8013
-   ConvolutionFilterScale -> 0x8014
-   ConvolutionFilterBias -> 0x8015
-   ConvolutionFormat -> 0x8017
-   ConvolutionWidth -> 0x8018
-   ConvolutionHeight -> 0x8019
-   MaxConvolutionWidth -> 0x801a
-   MaxConvolutionHeight -> 0x801b
+   ConvolutionBorderColor -> gl_CONVOLUTION_BORDER_COLOR
+   ConvolutionBorderMode -> gl_CONVOLUTION_BORDER_MODE
+   ConvolutionFilterScale -> gl_CONVOLUTION_FILTER_SCALE
+   ConvolutionFilterBias -> gl_CONVOLUTION_FILTER_BIAS
+   ConvolutionFormat -> gl_CONVOLUTION_FORMAT
+   ConvolutionWidth -> gl_CONVOLUTION_WIDTH
+   ConvolutionHeight -> gl_CONVOLUTION_HEIGHT
+   MaxConvolutionWidth -> gl_MAX_CONVOLUTION_WIDTH
+   MaxConvolutionHeight -> gl_MAX_CONVOLUTION_HEIGHT
 
 --------------------------------------------------------------------------------
 
@@ -219,8 +200,6 @@ getConvolutionParameteri f t p =
          (marshalConvolutionTarget t) (marshalConvolutionParameter p) buf
       peek1 f buf
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetConvolutionParameteriv,GLenum -> GLenum -> Ptr GLint -> IO ())
-
 --------------------------------------------------------------------------------
 
 data ConvolutionBorderMode' =
@@ -229,17 +208,18 @@ data ConvolutionBorderMode' =
    | ReplicateBorder'
 
 marshalConvolutionBorderMode' :: ConvolutionBorderMode' -> GLint
-marshalConvolutionBorderMode' x = case x of
-   Reduce' -> 0x8016
-   ConstantBorder' -> 0x8151
-   ReplicateBorder' -> 0x8153
+marshalConvolutionBorderMode' x = fromIntegral $ case x of
+   Reduce' -> gl_REDUCE
+   ConstantBorder' -> gl_CONSTANT_BORDER
+   ReplicateBorder' -> gl_REPLICATE_BORDER
 
 unmarshalConvolutionBorderMode' :: GLint -> ConvolutionBorderMode'
 unmarshalConvolutionBorderMode' x
-   | x == 0x8016 = Reduce'
-   | x == 0x8151 = ConstantBorder'
-   | x == 0x8153 = ReplicateBorder'
+   | y == gl_REDUCE = Reduce'
+   | y == gl_CONSTANT_BORDER = ConstantBorder'
+   | y == gl_REPLICATE_BORDER = ReplicateBorder'
    | otherwise = error ("unmarshalConvolutionBorderMode': illegal value " ++ show x)
+   where y = fromIntegral x
 
 --------------------------------------------------------------------------------
 
@@ -283,8 +263,6 @@ setConvolutionParameteri f t p x =
    glConvolutionParameteri
       (marshalConvolutionTarget t) (marshalConvolutionParameter p) (f x)
 
-EXTENSION_ENTRY("GL_ARB_imaging",glConvolutionParameteri,GLenum -> GLenum -> GLint -> IO ())
-
 --------------------------------------------------------------------------------
 
 convolutionFilterScale :: ConvolutionTarget -> StateVar (Color4 GLfloat)
@@ -303,16 +281,12 @@ getConvolutionParameterC4f ::
 getConvolutionParameterC4f t p =
    alloca $ \buf -> do
       glGetConvolutionParameterfv
-         (marshalConvolutionTarget t) (marshalConvolutionParameter p) buf
+         (marshalConvolutionTarget t) (marshalConvolutionParameter p) (castPtr buf)
       peek buf
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetConvolutionParameterfv,GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ())
-
 convolutionParameterC4f ::
    ConvolutionTarget -> ConvolutionParameter -> Color4 GLfloat -> IO ()
 convolutionParameterC4f t p c =
-   with c $
+   with c $ \ptr ->
       glConvolutionParameterfv
-         (marshalConvolutionTarget t) (marshalConvolutionParameter p)
-
-EXTENSION_ENTRY("GL_ARB_imaging",glConvolutionParameterfv,GLenum -> GLenum -> Ptr (Color4 GLfloat) -> IO ())
+         (marshalConvolutionTarget t) (marshalConvolutionParameter p) (castPtr ptr)
