@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.PixelRectangles.Histogram
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -18,34 +18,23 @@ module Graphics.Rendering.OpenGL.GL.PixelRectangles.Histogram (
    histogramRGBASizes, histogramLuminanceSize
 ) where
 
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Ptr ( Ptr )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   EnableCap(CapHistogram), makeStateVarMaybe )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLboolean, GLenum, GLint, GLsizei )
-import Graphics.Rendering.OpenGL.GL.Extensions (
-   FunPtr, unsafePerformIO, Invoker, getProcAddress )
-import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek1 )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.Rasterization (
-    PixelData(..) )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.ColorTable (
-   PixelInternalFormat )
-import Graphics.Rendering.OpenGL.GL.PixelData ( withPixelData )
-import Graphics.Rendering.OpenGL.GL.Texturing.PixelInternalFormat (
-   marshalPixelInternalFormat', unmarshalPixelInternalFormat )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.ColorTable ( Proxy(..) )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.Sink (
-   Sink(..), marshalSink, unmarshalSink )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.Reset (
-   Reset(..), marshalReset )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar, StateVar )
-import Graphics.Rendering.OpenGL.GL.VertexSpec ( Color4(..) )
-
---------------------------------------------------------------------------------
-
-#include "HsOpenGLExt.h"
+import Data.StateVar
+import Foreign.Marshal.Alloc
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.PeekPoke
+import Graphics.Rendering.OpenGL.GL.PixelData
+import Graphics.Rendering.OpenGL.GL.PixelRectangles.ColorTable
+import Graphics.Rendering.OpenGL.GL.PixelRectangles.Reset
+import Graphics.Rendering.OpenGL.GL.PixelRectangles.Sink
+import Graphics.Rendering.OpenGL.GL.Texturing.PixelInternalFormat
+import Graphics.Rendering.OpenGL.GL.VertexSpec
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glGetHistogram, glGetHistogramParameteriv, glHistogram, glResetHistogram,
+   gl_HISTOGRAM, gl_HISTOGRAM_ALPHA_SIZE, gl_HISTOGRAM_BLUE_SIZE,
+   gl_HISTOGRAM_FORMAT, gl_HISTOGRAM_GREEN_SIZE, gl_HISTOGRAM_LUMINANCE_SIZE,
+   gl_HISTOGRAM_RED_SIZE, gl_HISTOGRAM_SINK, gl_HISTOGRAM_WIDTH,
+   gl_PROXY_HISTOGRAM )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -55,8 +44,8 @@ data HistogramTarget =
 
 marshalHistogramTarget :: HistogramTarget -> GLenum
 marshalHistogramTarget x = case x of
-   Histogram -> 0x8024
-   ProxyHistogram -> 0x8025
+   Histogram -> gl_HISTOGRAM
+   ProxyHistogram -> gl_PROXY_HISTOGRAM
 
 proxyToHistogramTarget :: Proxy -> HistogramTarget
 proxyToHistogramTarget x = case x of
@@ -87,8 +76,6 @@ getHistogramParameteri f proxy p =
          buf
       peek1 f buf
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetHistogramParameteriv,GLenum -> GLenum -> Ptr GLint -> IO ())
-
 setHistogram :: Proxy -> (GLsizei, PixelInternalFormat, Sink) -> IO ()
 setHistogram proxy (w, int, sink) =
    glHistogram
@@ -96,8 +83,6 @@ setHistogram proxy (w, int, sink) =
       w
       (marshalPixelInternalFormat' int)
       (marshalSink sink)
-         
-EXTENSION_ENTRY("GL_ARB_imaging",glHistogram,GLenum -> GLsizei -> GLenum -> GLboolean -> IO ())
 
 --------------------------------------------------------------------------------
 
@@ -108,15 +93,11 @@ getHistogram reset pd =
          (marshalHistogramTarget Histogram)
          (marshalReset reset)
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetHistogram,GLenum -> GLboolean -> GLenum -> GLenum -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 resetHistogram :: IO ()
 resetHistogram = glResetHistogram (marshalHistogramTarget Histogram)
 
-EXTENSION_ENTRY("GL_ARB_imaging",glResetHistogram,GLenum -> IO ())
-
 --------------------------------------------------------------------------------
 
 data GetHistogramParameterPName =
@@ -131,14 +112,14 @@ data GetHistogramParameterPName =
 
 marshalGetHistogramParameterPName :: GetHistogramParameterPName -> GLenum
 marshalGetHistogramParameterPName x = case x of
-   HistogramWidth -> 0x8026
-   HistogramFormat -> 0x8027
-   HistogramRedSize -> 0x8028
-   HistogramGreenSize -> 0x8029
-   HistogramBlueSize -> 0x802a
-   HistogramAlphaSize -> 0x802b
-   HistogramLuminanceSize -> 0x802c
-   HistogramSink -> 0x802d
+   HistogramWidth -> gl_HISTOGRAM_WIDTH
+   HistogramFormat -> gl_HISTOGRAM_FORMAT
+   HistogramRedSize -> gl_HISTOGRAM_RED_SIZE
+   HistogramGreenSize -> gl_HISTOGRAM_GREEN_SIZE
+   HistogramBlueSize -> gl_HISTOGRAM_BLUE_SIZE
+   HistogramAlphaSize -> gl_HISTOGRAM_ALPHA_SIZE
+   HistogramLuminanceSize -> gl_HISTOGRAM_LUMINANCE_SIZE
+   HistogramSink -> gl_HISTOGRAM_SINK
 
 --------------------------------------------------------------------------------
 
@@ -154,4 +135,4 @@ histogramRGBASizes proxy =
 histogramLuminanceSize :: Proxy -> GettableStateVar GLsizei
 histogramLuminanceSize proxy =
    makeGettableStateVar $
-      getHistogramParameteri id proxy HistogramLuminanceSize
+      getHistogramParameteri fromIntegral proxy HistogramLuminanceSize
