@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.PixelRectangles.Minmax
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -17,32 +17,18 @@ module Graphics.Rendering.OpenGL.GL.PixelRectangles.Minmax (
    minmax, getMinmax, resetMinmax
 ) where
 
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Ptr ( Ptr )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   EnableCap(CapMinmax), makeStateVarMaybe )
-import Graphics.Rendering.OpenGL.GL.BasicTypes ( GLboolean, GLenum, GLint )
-import Graphics.Rendering.OpenGL.GL.Extensions (
-   FunPtr, unsafePerformIO, Invoker, getProcAddress )
-import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek1 )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.ColorTable (
-   PixelInternalFormat )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.Rasterization (
-    PixelData(..) )
-import Graphics.Rendering.OpenGL.GL.PixelData ( withPixelData )
-import Graphics.Rendering.OpenGL.GL.Texturing.PixelInternalFormat (
-   marshalPixelInternalFormat', unmarshalPixelInternalFormat )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.Histogram (
-   Reset(..), Sink )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.Reset (
-   marshalReset )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles.Sink (
-   marshalSink, unmarshalSink )
-import Graphics.Rendering.OpenGL.GL.StateVar ( StateVar )
-
---------------------------------------------------------------------------------
-
-#include "HsOpenGLExt.h"
+import Data.StateVar
+import Foreign.Marshal.Alloc
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.PeekPoke
+import Graphics.Rendering.OpenGL.GL.PixelData
+import Graphics.Rendering.OpenGL.GL.PixelRectangles.Reset
+import Graphics.Rendering.OpenGL.GL.PixelRectangles.Sink
+import Graphics.Rendering.OpenGL.GL.Texturing.PixelInternalFormat
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glGetMinmax, glGetMinmaxParameteriv, glMinmax, glResetMinmax, gl_MINMAX,
+   gl_MINMAX_FORMAT, gl_MINMAX_SINK )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -51,7 +37,7 @@ data MinmaxTarget =
 
 marshalMinmaxTarget :: MinmaxTarget -> GLenum
 marshalMinmaxTarget x = case x of
-   Minmax -> 0x802e
+   Minmax -> gl_MINMAX
 
 --------------------------------------------------------------------------------
 
@@ -71,8 +57,6 @@ setMinmax (int, sink) =
       (marshalPixelInternalFormat' int)
       (marshalSink sink)
 
-EXTENSION_ENTRY("GL_ARB_imaging",glMinmax,GLenum -> GLenum -> GLboolean -> IO ())
-
 --------------------------------------------------------------------------------
 
 getMinmax :: Reset -> PixelData a -> IO ()
@@ -80,15 +64,11 @@ getMinmax reset pd =
    withPixelData pd $
       glGetMinmax (marshalMinmaxTarget Minmax) (marshalReset reset)
 
-EXTENSION_ENTRY("GL_ARB_imaging",glGetMinmax,GLenum -> GLboolean -> GLenum -> GLenum -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 resetMinmax :: IO ()
 resetMinmax = glResetMinmax (marshalMinmaxTarget Minmax)
 
-EXTENSION_ENTRY("GL_ARB_imaging",glResetMinmax,GLenum -> IO ())
-
 --------------------------------------------------------------------------------
 
 data GetMinmaxParameterPName =
@@ -97,8 +77,8 @@ data GetMinmaxParameterPName =
 
 marshalGetMinmaxParameterPName :: GetMinmaxParameterPName -> GLenum
 marshalGetMinmaxParameterPName x = case x of
-   MinmaxFormat -> 0x802f
-   MinmaxSink -> 0x8030
+   MinmaxFormat -> gl_MINMAX_FORMAT
+   MinmaxSink -> gl_MINMAX_SINK
 
 --------------------------------------------------------------------------------
 
@@ -110,5 +90,3 @@ getMinmaxParameteri f p =
          (marshalGetMinmaxParameterPName p)
          buf
       peek1 f buf
-
-EXTENSION_ENTRY("GL_ARB_imaging",glGetMinmaxParameteriv,GLenum -> GLenum -> Ptr GLint -> IO ())
