@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.PixelRectangles.PixelMap
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -13,33 +13,28 @@
 --
 --------------------------------------------------------------------------------
 
+{-# LANGUAGE TypeSynonymInstances #-}
+
 module Graphics.Rendering.OpenGL.GL.PixelRectangles.PixelMap (
    PixelMapTarget(..), PixelMapComponent, PixelMap(..), GLpixelmap,
    maxPixelMapTable, pixelMap, pixelMapIToRGBA, pixelMapRGBAToRGBA,
 ) where
 
-import Data.List ( zipWith4 )
-import Data.Word
-import Foreign.ForeignPtr ( ForeignPtr, mallocForeignPtrArray, withForeignPtr )
-import Foreign.Marshal.Array ( allocaArray, peekArray, pokeArray, withArrayLen )
-import Foreign.Ptr ( Ptr )
-import Foreign.Storable ( Storable(..) )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLenum, GLushort, GLuint, GLsizei, GLfloat )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetMaxPixelMapTable,GetPixelMapIToISize,GetPixelMapSToSSize,
-            GetPixelMapIToRSize,GetPixelMapIToGSize,GetPixelMapIToBSize,
-            GetPixelMapIToASize,GetPixelMapRToRSize,GetPixelMapGToGSize,
-            GetPixelMapBToBSize,GetPixelMapAToASize),
-   getInteger1, getSizei1 )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   HasSetter(($=)), HasGetter(get), GettableStateVar, makeGettableStateVar,
-   StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.VertexSpec ( Color4(..) )
-
---------------------------------------------------------------------------------
-
-#include "HsOpenGLTypes.h"
+import Data.List
+import Data.StateVar
+import Foreign.ForeignPtr
+import Foreign.Marshal.Array
+import Foreign.Ptr
+import Foreign.Storable
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.VertexSpec
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glGetPixelMapfv, glGetPixelMapuiv, glGetPixelMapusv, glPixelMapfv,
+   glPixelMapuiv, glPixelMapusv, gl_PIXEL_MAP_A_TO_A, gl_PIXEL_MAP_B_TO_B,
+   gl_PIXEL_MAP_G_TO_G, gl_PIXEL_MAP_I_TO_A, gl_PIXEL_MAP_I_TO_B,
+   gl_PIXEL_MAP_I_TO_G, gl_PIXEL_MAP_I_TO_I, gl_PIXEL_MAP_I_TO_R,
+   gl_PIXEL_MAP_R_TO_R, gl_PIXEL_MAP_S_TO_S )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -57,18 +52,18 @@ data PixelMapTarget =
 
 marshalPixelMapTarget :: PixelMapTarget -> GLenum
 marshalPixelMapTarget x = case x of
-   IToI -> 0xc70
-   SToS -> 0xc71
-   IToR -> 0xc72
-   IToG -> 0xc73
-   IToB -> 0xc74
-   IToA -> 0xc75
-   RToR -> 0xc76
-   GToG -> 0xc77
-   BToB -> 0xc78
-   AToA -> 0xc79
+   IToI -> gl_PIXEL_MAP_I_TO_I
+   SToS -> gl_PIXEL_MAP_S_TO_S
+   IToR -> gl_PIXEL_MAP_I_TO_R
+   IToG -> gl_PIXEL_MAP_I_TO_G
+   IToB -> gl_PIXEL_MAP_I_TO_B
+   IToA -> gl_PIXEL_MAP_I_TO_A
+   RToR -> gl_PIXEL_MAP_R_TO_R
+   GToG -> gl_PIXEL_MAP_G_TO_G
+   BToB -> gl_PIXEL_MAP_B_TO_B
+   AToA -> gl_PIXEL_MAP_A_TO_A
 
-pixelMapTargetToGetPName :: PixelMapTarget -> GetPName
+pixelMapTargetToGetPName :: PixelMapTarget -> PName1I
 pixelMapTargetToGetPName x = case x of
    IToI -> GetPixelMapIToISize
    SToS -> GetPixelMapSToSSize
@@ -92,36 +87,18 @@ class Storable c => PixelMapComponent c where
    getPixelMapv :: GLenum -> Ptr c -> IO ()
    pixelMapv :: GLenum -> GLsizei -> Ptr c -> IO ()
 
-instance PixelMapComponent GLushort_ where
+instance PixelMapComponent GLushort where
    getPixelMapv = glGetPixelMapusv
    pixelMapv = glPixelMapusv
 
-foreign import CALLCONV unsafe "glGetPixelMapusv" glGetPixelMapusv ::
-   GLenum -> Ptr GLushort -> IO ()
-
-foreign import CALLCONV unsafe "glPixelMapusv" glPixelMapusv ::
-   GLenum -> GLsizei -> Ptr GLushort -> IO ()
-
-instance PixelMapComponent GLuint_ where
+instance PixelMapComponent GLuint where
    getPixelMapv = glGetPixelMapuiv
    pixelMapv = glPixelMapuiv
 
-foreign import CALLCONV unsafe "glGetPixelMapuiv" glGetPixelMapuiv ::
-   GLenum -> Ptr GLuint -> IO ()
-
-foreign import CALLCONV unsafe "glPixelMapuiv" glPixelMapuiv ::
-   GLenum -> GLsizei -> Ptr GLuint -> IO ()
-
-instance PixelMapComponent GLfloat_ where
+instance PixelMapComponent GLfloat where
    getPixelMapv = glGetPixelMapfv
    pixelMapv = glPixelMapfv
 
-foreign import CALLCONV unsafe "glGetPixelMapfv" glGetPixelMapfv ::
-   GLenum -> Ptr GLfloat -> IO ()
-
-foreign import CALLCONV unsafe "glPixelMapfv" glPixelMapfv ::
-   GLenum -> GLsizei -> Ptr GLfloat -> IO ()
-
 --------------------------------------------------------------------------------
 
 class PixelMap m where
@@ -151,14 +128,7 @@ class PixelMap m where
 --------------------------------------------------------------------------------
 
 data GLpixelmap a = GLpixelmap Int (ForeignPtr a)
-#ifdef __HADDOCK__
--- Help Haddock a bit, because it doesn't do any instance inference.
-instance Eq (GLpixelmap a)
-instance Ord (GLpixelmap a)
-instance Show (GLpixelmap a)
-#else
    deriving ( Eq, Ord, Show )
-#endif
 
 instance PixelMap GLpixelmap where
    withNewPixelMap size f = do
