@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.PixelRectangles.PixelStorage
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -18,17 +18,10 @@ module Graphics.Rendering.OpenGL.GL.PixelRectangles.PixelStorage (
    skipPixels, rowAlignment, imageHeight, skipImages
 ) where
 
-import Graphics.Rendering.OpenGL.GL.BasicTypes ( GLenum, GLint )
-import Graphics.Rendering.OpenGL.GL.GLboolean (
-   marshalGLboolean, unmarshalGLboolean )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetUnpackSwapBytes,GetUnpackLSBFirst,GetUnpackRowLength,
-            GetUnpackSkipRows,GetUnpackSkipPixels,GetUnpackAlignment,
-            GetUnpackImageHeight,GetUnpackSkipImages,GetPackSwapBytes,
-            GetPackLSBFirst,GetPackRowLength,GetPackSkipRows,GetPackSkipPixels,
-            GetPackAlignment,GetPackImageHeight,GetPackSkipImages),
-   getBoolean1, getInteger1 )
-import Graphics.Rendering.OpenGL.GL.StateVar ( StateVar, makeStateVar )
+import Data.StateVar
+import Graphics.Rendering.OpenGL.GL.GLboolean
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -59,22 +52,22 @@ data PixelStore =
 
 marshalPixelStore :: PixelStore -> GLenum
 marshalPixelStore x = case x of
-   UnpackSwapBytes -> 0xcf0
-   UnpackLSBFirst -> 0xcf1
-   UnpackRowLength -> 0xcf2
-   UnpackSkipRows -> 0xcf3
-   UnpackSkipPixels -> 0xcf4
-   UnpackAlignment -> 0xcf5
-   PackSwapBytes -> 0xd00
-   PackLSBFirst -> 0xd01
-   PackRowLength -> 0xd02
-   PackSkipRows -> 0xd03
-   PackSkipPixels -> 0xd04
-   PackAlignment -> 0xd05
-   PackSkipImages -> 0x806b
-   PackImageHeight -> 0x806c
-   UnpackSkipImages -> 0x806d
-   UnpackImageHeight -> 0x806e
+   UnpackSwapBytes -> gl_UNPACK_SWAP_BYTES
+   UnpackLSBFirst -> gl_UNPACK_LSB_FIRST
+   UnpackRowLength -> gl_UNPACK_ROW_LENGTH
+   UnpackSkipRows -> gl_UNPACK_SKIP_ROWS
+   UnpackSkipPixels -> gl_UNPACK_SKIP_PIXELS
+   UnpackAlignment -> gl_UNPACK_ALIGNMENT
+   PackSwapBytes -> gl_PACK_SWAP_BYTES
+   PackLSBFirst -> gl_PACK_LSB_FIRST
+   PackRowLength -> gl_PACK_ROW_LENGTH
+   PackSkipRows -> gl_PACK_SKIP_ROWS
+   PackSkipPixels -> gl_PACK_SKIP_PIXELS
+   PackAlignment -> gl_PACK_ALIGNMENT
+   PackSkipImages -> gl_PACK_SKIP_IMAGES
+   PackImageHeight -> gl_PACK_IMAGE_HEIGHT
+   UnpackSkipImages -> gl_UNPACK_SKIP_IMAGES
+   UnpackImageHeight -> gl_UNPACK_IMAGE_HEIGHT
 
 --------------------------------------------------------------------------------
 
@@ -112,17 +105,14 @@ skipImages Unpack = pixelStorei GetUnpackSkipImages UnpackSkipImages
 
 --------------------------------------------------------------------------------
 
-pixelStoreb :: GetPName -> PixelStore -> StateVar Bool
+pixelStoreb :: PName1I -> PixelStore -> StateVar Bool
 pixelStoreb pn ps =
    makeStateVar
       (getBoolean1 unmarshalGLboolean pn)
       (glPixelStorei (marshalPixelStore ps) . marshalGLboolean)
 
-pixelStorei :: GetPName -> PixelStore -> StateVar GLint
+pixelStorei :: PName1I -> PixelStore -> StateVar GLint
 pixelStorei pn ps =
    makeStateVar
       (getInteger1 id pn)
       (glPixelStorei (marshalPixelStore ps))
-
-foreign import CALLCONV unsafe "glPixelStorei" glPixelStorei ::
-   GLenum -> GLint -> IO ()
