@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.PixelRectangles.PixelTransfer
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -19,25 +19,24 @@ module Graphics.Rendering.OpenGL.GL.PixelRectangles.PixelTransfer (
    rgbaScale, rgbaBias
 ) where
 
-import Graphics.Rendering.OpenGL.GL.Capability (
-   marshalCapability, unmarshalCapability )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLenum, GLint, GLfloat, Capability )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetMapColor,GetMapStencil,GetIndexShift,GetIndexOffset,
-            GetRedScale,GetGreenScale,GetBlueScale,GetAlphaScale,GetDepthScale,
-            GetRedBias,GetGreenBias,GetBlueBias,GetAlphaBias,GetDepthBias,
-            GetPostConvolutionRedScale,GetPostConvolutionGreenScale,
-            GetPostConvolutionBlueScale,GetPostConvolutionAlphaScale,
-            GetPostConvolutionRedBias,GetPostConvolutionGreenBias,
-            GetPostConvolutionBlueBias,GetPostConvolutionAlphaBias,
-            GetPostColorMatrixRedScale,GetPostColorMatrixGreenScale,
-            GetPostColorMatrixBlueScale,GetPostColorMatrixAlphaScale,
-            GetPostColorMatrixRedBias,GetPostColorMatrixGreenBias,
-            GetPostColorMatrixBlueBias,GetPostColorMatrixAlphaBias),
-   getBoolean1, getInteger1, getFloat1 )
-import Graphics.Rendering.OpenGL.GL.StateVar ( StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.VertexSpec ( Color4(..) )
+import Data.StateVar
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.VertexSpec
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glPixelTransferf, glPixelTransferi, gl_ALPHA_BIAS, gl_ALPHA_SCALE,
+   gl_BLUE_BIAS, gl_BLUE_SCALE, gl_DEPTH_BIAS, gl_DEPTH_SCALE, gl_GREEN_BIAS,
+   gl_GREEN_SCALE, gl_INDEX_OFFSET, gl_INDEX_SHIFT, gl_MAP_COLOR,
+   gl_MAP_STENCIL, gl_POST_COLOR_MATRIX_ALPHA_BIAS,
+   gl_POST_COLOR_MATRIX_ALPHA_SCALE, gl_POST_COLOR_MATRIX_BLUE_BIAS,
+   gl_POST_COLOR_MATRIX_BLUE_SCALE, gl_POST_COLOR_MATRIX_GREEN_BIAS,
+   gl_POST_COLOR_MATRIX_GREEN_SCALE, gl_POST_COLOR_MATRIX_RED_BIAS,
+   gl_POST_COLOR_MATRIX_RED_SCALE, gl_POST_CONVOLUTION_ALPHA_BIAS,
+   gl_POST_CONVOLUTION_ALPHA_SCALE, gl_POST_CONVOLUTION_BLUE_BIAS,
+   gl_POST_CONVOLUTION_BLUE_SCALE, gl_POST_CONVOLUTION_GREEN_BIAS,
+   gl_POST_CONVOLUTION_GREEN_SCALE, gl_POST_CONVOLUTION_RED_BIAS,
+   gl_POST_CONVOLUTION_RED_SCALE, gl_RED_BIAS, gl_RED_SCALE )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -75,36 +74,36 @@ data PixelTransfer =
 
 marshalPixelTransfer :: PixelTransfer -> GLenum
 marshalPixelTransfer x = case x of
-   MapColor -> 0xd10
-   MapStencil -> 0xd11
-   IndexShift -> 0xd12
-   IndexOffset -> 0xd13
-   RedScale -> 0xd14
-   RedBias -> 0xd15
-   GreenScale -> 0xd18
-   GreenBias -> 0xd19
-   BlueScale -> 0xd1a
-   BlueBias -> 0xd1b
-   AlphaScale -> 0xd1c
-   AlphaBias -> 0xd1d
-   DepthScale -> 0xd1e
-   DepthBias -> 0xd1f
-   PostConvolutionRedScale -> 0x801c
-   PostConvolutionGreenScale -> 0x801d
-   PostConvolutionBlueScale -> 0x801e
-   PostConvolutionAlphaScale -> 0x801f
-   PostConvolutionRedBias -> 0x8020
-   PostConvolutionGreenBias -> 0x8021
-   PostConvolutionBlueBias -> 0x8022
-   PostConvolutionAlphaBias -> 0x8023
-   PostColorMatrixRedScale -> 0x80b4
-   PostColorMatrixGreenScale -> 0x80b5
-   PostColorMatrixBlueScale -> 0x80b6
-   PostColorMatrixAlphaScale -> 0x80b7
-   PostColorMatrixRedBias -> 0x80b8
-   PostColorMatrixGreenBias -> 0x80b9
-   PostColorMatrixBlueBias -> 0x80ba
-   PostColorMatrixAlphaBias -> 0x80bb
+   MapColor -> gl_MAP_COLOR
+   MapStencil -> gl_MAP_STENCIL
+   IndexShift -> gl_INDEX_SHIFT
+   IndexOffset -> gl_INDEX_OFFSET
+   RedScale -> gl_RED_SCALE
+   RedBias -> gl_RED_BIAS
+   GreenScale -> gl_GREEN_SCALE
+   GreenBias -> gl_GREEN_BIAS
+   BlueScale -> gl_BLUE_SCALE
+   BlueBias -> gl_BLUE_BIAS
+   AlphaScale -> gl_ALPHA_SCALE
+   AlphaBias -> gl_ALPHA_BIAS
+   DepthScale -> gl_DEPTH_SCALE
+   DepthBias -> gl_DEPTH_BIAS
+   PostConvolutionRedScale -> gl_POST_CONVOLUTION_RED_SCALE
+   PostConvolutionGreenScale -> gl_POST_CONVOLUTION_GREEN_SCALE
+   PostConvolutionBlueScale -> gl_POST_CONVOLUTION_BLUE_SCALE
+   PostConvolutionAlphaScale -> gl_POST_CONVOLUTION_ALPHA_SCALE
+   PostConvolutionRedBias -> gl_POST_CONVOLUTION_RED_BIAS
+   PostConvolutionGreenBias -> gl_POST_CONVOLUTION_GREEN_BIAS
+   PostConvolutionBlueBias -> gl_POST_CONVOLUTION_BLUE_BIAS
+   PostConvolutionAlphaBias -> gl_POST_CONVOLUTION_ALPHA_BIAS
+   PostColorMatrixRedScale -> gl_POST_COLOR_MATRIX_RED_SCALE
+   PostColorMatrixGreenScale -> gl_POST_COLOR_MATRIX_GREEN_SCALE
+   PostColorMatrixBlueScale -> gl_POST_COLOR_MATRIX_BLUE_SCALE
+   PostColorMatrixAlphaScale -> gl_POST_COLOR_MATRIX_ALPHA_SCALE
+   PostColorMatrixRedBias -> gl_POST_COLOR_MATRIX_RED_BIAS
+   PostColorMatrixGreenBias -> gl_POST_COLOR_MATRIX_GREEN_BIAS
+   PostColorMatrixBlueBias -> gl_POST_COLOR_MATRIX_BLUE_BIAS
+   PostColorMatrixAlphaBias -> gl_POST_COLOR_MATRIX_ALPHA_BIAS
 
 --------------------------------------------------------------------------------
 
@@ -116,7 +115,7 @@ data PixelTransferStage =
 
 stageToGetScales ::
       PixelTransferStage
-   -> (GetPName, GetPName, GetPName, GetPName)
+   -> (PName1F, PName1F, PName1F, PName1F)
 stageToGetScales s = case s of
    PreConvolution  -> (GetRedScale,
                        GetGreenScale,
@@ -150,7 +149,7 @@ stageToSetScales s = case s of
 
 stageToGetBiases ::
       PixelTransferStage
-   -> (GetPName, GetPName, GetPName, GetPName)
+   -> (PName1F, PName1F, PName1F, PName1F)
 stageToGetBiases s = case s of
    PreConvolution  -> (GetRedBias,
                        GetGreenBias,
@@ -210,30 +209,27 @@ rgbaBias s = pixelTransfer4f (stageToGetBiases s) (stageToSetBiases s)
 
 --------------------------------------------------------------------------------
 
-pixelTransferb :: GetPName -> PixelTransfer -> StateVar Capability
+pixelTransferb :: GetPName1I p => p -> PixelTransfer -> StateVar Capability
 pixelTransferb pn pt =
    makeStateVar
       (getBoolean1 unmarshalCapability pn)
       (glPixelTransferi (marshalPixelTransfer pt) .
        fromIntegral . marshalCapability)
 
-pixelTransferi :: GetPName -> PixelTransfer -> StateVar GLint
+pixelTransferi :: GetPName1I p => p -> PixelTransfer -> StateVar GLint
 pixelTransferi pn pt =
    makeStateVar
       (getInteger1 id pn)
       (glPixelTransferi (marshalPixelTransfer pt))
 
-foreign import CALLCONV unsafe "glPixelTransferi" glPixelTransferi ::
-   GLenum -> GLint -> IO ()
-
-pixelTransferf :: GetPName -> PixelTransfer -> StateVar GLfloat
+pixelTransferf :: GetPName1F p => p -> PixelTransfer -> StateVar GLfloat
 pixelTransferf pn pt =
    makeStateVar
       (getFloat1 id pn)
       (glPixelTransferf (marshalPixelTransfer pt))
 
-pixelTransfer4f ::
-      (GetPName,      GetPName,      GetPName,      GetPName)
+pixelTransfer4f :: GetPName1F p =>
+      (p, p, p, p)
    -> (PixelTransfer, PixelTransfer, PixelTransfer, PixelTransfer)
    -> StateVar (Color4 GLfloat)
 pixelTransfer4f (pr, pg, pb, pa) (tr, tg, tb, ta) = makeStateVar get4f set4f
@@ -248,6 +244,3 @@ pixelTransfer4f (pr, pg, pb, pa) (tr, tg, tb, ta) = makeStateVar get4f set4f
             glPixelTransferf (marshalPixelTransfer tg) g
             glPixelTransferf (marshalPixelTransfer tb) b
             glPixelTransferf (marshalPixelTransfer ta) a
-
-foreign import CALLCONV unsafe "glPixelTransferf" glPixelTransferf ::
-   GLenum -> GLfloat -> IO ()
