@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.ReadCopyPixels
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -18,21 +18,21 @@ module Graphics.Rendering.OpenGL.GL.ReadCopyPixels (
    readPixels, readBuffer,
 
    -- * Copying Pixels
-   PixelCopyType(..), copyPixels
+   PixelCopyType(..), copyPixels,
+
+   -- * Copying Pixels for framebuffers
+   BlitFramebufferMask(..), blitFramebuffer
 ) where
 
-import Foreign.Ptr ( Ptr )
-import Graphics.Rendering.OpenGL.GL.BasicTypes ( GLenum, GLint, GLsizei )
-import Graphics.Rendering.OpenGL.GL.BufferMode (
-   marshalBufferMode, unmarshalBufferMode )
-import Graphics.Rendering.OpenGL.GL.Framebuffer ( BufferMode(..) )
-import Graphics.Rendering.OpenGL.GL.CoordTrans ( Position(..), Size(..) )
-import Graphics.Rendering.OpenGL.GL.PixelData ( withPixelData )
-import Graphics.Rendering.OpenGL.GL.PixelRectangles ( PixelData )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetReadBuffer), getEnum1 )
-import Graphics.Rendering.OpenGL.GL.StateVar ( StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal ( recordInvalidValue )
+import Data.StateVar
+import Graphics.Rendering.OpenGL.GL.BufferMode
+import Graphics.Rendering.OpenGL.GL.CoordTrans
+import Graphics.Rendering.OpenGL.GL.PixelData
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.Texturing.Parameters
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility ( glCopyPixels )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -40,9 +40,6 @@ readPixels :: Position -> Size -> PixelData a -> IO ()
 readPixels (Position x y) (Size w h) pd =
    withPixelData pd $ glReadPixels x y w h
 
-foreign import CALLCONV unsafe "glReadPixels" glReadPixels ::
-   GLint -> GLint -> GLsizei -> GLsizei -> GLenum -> GLenum -> Ptr a -> IO ()
-
 --------------------------------------------------------------------------------
 
 readBuffer :: StateVar BufferMode
@@ -51,8 +48,6 @@ readBuffer =
       (getEnum1 unmarshalBufferMode GetReadBuffer)
       (maybe recordInvalidValue glReadBuffer . marshalBufferMode)
 
-foreign import CALLCONV unsafe "glReadBuffer" glReadBuffer :: GLenum -> IO ()
-
 --------------------------------------------------------------------------------
 
 data PixelCopyType =
@@ -63,9 +58,9 @@ data PixelCopyType =
 
 marshalPixelCopyType :: PixelCopyType -> GLenum
 marshalPixelCopyType x = case x of
-   CopyColor -> 0x1800
-   CopyDepth -> 0x1801
-   CopyStencil -> 0x1802
+   CopyColor -> gl_COLOR
+   CopyDepth -> gl_DEPTH
+   CopyStencil -> gl_STENCIL
 
 --------------------------------------------------------------------------------
 
@@ -73,5 +68,25 @@ copyPixels :: Position -> Size -> PixelCopyType -> IO ()
 copyPixels (Position x y) (Size w h) t =
    glCopyPixels x y w h (marshalPixelCopyType t)
 
-foreign import CALLCONV unsafe "glCopyPixels" glCopyPixels ::
-   GLint -> GLint -> GLsizei -> GLsizei -> GLenum -> IO ()
+--------------------------------------------------------------------------------
+
+data BlitFramebufferMask
+   = BlitColorBuffer
+   | BlitStencilBuffer
+   | BlitDepthBuffer
+
+marshalBlitFramebufferMask :: [BlitFramebufferMask] -> GLbitfield
+marshalBlitFramebufferMask = fromIntegral . sum . map marshal
+    where marshal x = case x of
+             BlitColorBuffer -> gl_COLOR_BUFFER_BIT
+             BlitStencilBuffer -> gl_STENCIL_BUFFER_BIT
+             BlitDepthBuffer -> gl_DEPTH_BUFFER_BIT
+
+blitFramebuffer :: Position -> Position -> Position -> Position -> [BlitFramebufferMask]
+   -> TextureFilter -> IO ()
+blitFramebuffer (Position sx0 sy0) (Position sx1 sy1) (Position dx0 dy0) (Position dx1 dy1)
+   bfbm filt = glBlitFramebuffer sx0 sy0 sx1 sy1 dx0 dy0 dx1 dy1
+      (marshalBlitFramebufferMask bfbm) (marshalTextureFilter filt)
+      where marshalTextureFilter x = case x of
+                Nearest -> gl_NEAREST
+                Linear' -> gl_LINEAR
