@@ -4,7 +4,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.RenderMode
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -17,12 +17,12 @@ module Graphics.Rendering.OpenGL.GL.RenderMode (
    RenderMode(..), withRenderMode, renderMode
 ) where
 
-import Graphics.Rendering.OpenGL.GL.BasicTypes ( GLenum, GLint )
-import Graphics.Rendering.OpenGL.GL.Exception ( finallyRet )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetRenderMode), getEnum1 )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   HasGetter(get), GettableStateVar, makeGettableStateVar )
+import Data.StateVar
+import Graphics.Rendering.OpenGL.GL.Exception
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glRenderMode, gl_FEEDBACK, gl_RENDER, gl_SELECT )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -34,15 +34,15 @@ data RenderMode =
 
 marshalRenderMode :: RenderMode -> GLenum
 marshalRenderMode x = case x of
-   Render -> 0x1c00
-   Feedback -> 0x1c01
-   Select -> 0x1c02
+   Render -> gl_RENDER
+   Feedback -> gl_FEEDBACK
+   Select -> gl_SELECT
 
 unmarshalRenderMode :: GLenum -> RenderMode
 unmarshalRenderMode x
-   | x == 0x1c00 = Render
-   | x == 0x1c01 = Feedback
-   | x == 0x1c02 = Select
+   | x == gl_RENDER = Render
+   | x == gl_FEEDBACK = Feedback
+   | x == gl_SELECT = Select
    | otherwise = error ("unmarshalRenderMode: illegal value " ++ show x)
 
 --------------------------------------------------------------------------------
@@ -50,14 +50,12 @@ unmarshalRenderMode x
 withRenderMode :: RenderMode -> IO a -> IO (a, GLint)
 withRenderMode newMode action = do
    oldMode <- get renderMode
-   setRenderMode newMode
+   _ <- setRenderMode newMode
    action `finallyRet` setRenderMode oldMode
 
 setRenderMode :: RenderMode -> IO GLint
 setRenderMode = glRenderMode . marshalRenderMode
 
-foreign import CALLCONV unsafe "glRenderMode" glRenderMode :: GLenum -> IO GLint
-
 --------------------------------------------------------------------------------
 
 renderMode :: GettableStateVar RenderMode
