@@ -17,14 +17,9 @@ module Graphics.Rendering.OpenGL.GL.Texturing.Application (
    texture
 ) where
 
-import Graphics.Rendering.OpenGL.GL.BasicTypes ( Capability )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   EnableCap(CapTexture1D,CapTexture2D,CapTexture3D,CapTextureCubeMap,
-             CapTextureRectangle),
-   makeCapability )
-import Graphics.Rendering.OpenGL.GL.StateVar ( StateVar )
-import Graphics.Rendering.OpenGL.GL.Texturing.Specification (
-   TextureTarget(..) )
+import Data.StateVar
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.Texturing.Specification
 
 --------------------------------------------------------------------------------
 
