@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.Texturing.Objects
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -19,25 +19,18 @@ module Graphics.Rendering.OpenGL.GL.Texturing.Objects (
    TexturePriority, texturePriority, prioritizeTextures
 ) where
 
-import Data.List ( partition )
-import Foreign.Marshal.Array ( withArray, withArrayLen, peekArray, allocaArray )
-import Foreign.Ptr ( Ptr )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLboolean, GLuint, GLsizei, GLenum, GLclampf )
-import Graphics.Rendering.OpenGL.GL.BufferObjects ( ObjectName(..) )
-import Graphics.Rendering.OpenGL.GL.GLboolean ( unmarshalGLboolean )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetTextureBinding1D,GetTextureBinding2D,GetTextureBinding3D,
-            GetTextureBindingCubeMap,GetTextureBindingRectangle),
-   getEnum1)
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar, StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.Texturing.TexParameter (
-   TexParameter(TextureResident,TexturePriority), texParamf, getTexParameteri )
-import Graphics.Rendering.OpenGL.GL.Texturing.Specification (
-   TextureTarget(..) )
-import Graphics.Rendering.OpenGL.GL.Texturing.TextureTarget (
-   marshalTextureTarget )
+import Data.List
+import Data.Maybe (fromMaybe)
+import Data.ObjectName
+import Data.StateVar
+import Foreign.Marshal.Array
+import Graphics.Rendering.OpenGL.GL.GLboolean
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.Texturing.TexParameter
+import Graphics.Rendering.OpenGL.GL.Texturing.TextureTarget
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+ glAreTexturesResident, glPrioritizeTextures )
+import Graphics.Rendering.OpenGL.Raw.Core31
 
 --------------------------------------------------------------------------------
 
@@ -58,28 +51,19 @@ instance ObjectName TextureObject where
 
    isObjectName = fmap unmarshalGLboolean . glIsTexture . textureID
 
-foreign import CALLCONV unsafe "glGenTextures"
-   glGenTextures :: GLsizei -> Ptr GLuint -> IO ()
-
-foreign import CALLCONV unsafe "glDeleteTextures"
-   glDeleteTextures :: GLsizei -> Ptr GLuint -> IO ()
-
-foreign import CALLCONV unsafe "glIsTexture"
-   glIsTexture :: GLuint -> IO GLboolean
-
 --------------------------------------------------------------------------------
 
 textureBinding :: TextureTarget -> StateVar (Maybe TextureObject)
 textureBinding t =
    makeStateVar
-      (do o <- getEnum1 TextureObject (textureTargetToGetPName t)
+      (do o <- getEnum1 (TextureObject . fromIntegral) (textureTargetToGetPName t)
           return $ if o == defaultTextureObject then Nothing else Just o)
-      (glBindTexture (marshalTextureTarget t) . textureID . (maybe defaultTextureObject id))
+      (glBindTexture (marshalTextureTarget t) . textureID . (fromMaybe defaultTextureObject))
 
 defaultTextureObject :: TextureObject
 defaultTextureObject = TextureObject 0
 
-textureTargetToGetPName :: TextureTarget -> GetPName
+textureTargetToGetPName :: TextureTarget -> PName1I
 textureTargetToGetPName x = case x of
     Texture1D -> GetTextureBinding1D
     Texture2D -> GetTextureBinding2D
@@ -87,9 +71,6 @@ textureTargetToGetPName x = case x of
     TextureCubeMap -> GetTextureBindingCubeMap
     TextureRectangle -> GetTextureBindingRectangle
 
-foreign import CALLCONV unsafe "glBindTexture"
-   glBindTexture :: GLenum -> GLuint -> IO ()
-
 --------------------------------------------------------------------------------
 
 textureResident :: TextureTarget -> GettableStateVar Bool
@@ -110,21 +91,15 @@ areTexturesResident texObjs = do
                let (resident, nonResident) = partition (unmarshalGLboolean . snd) tr
                return (map fst resident, map fst nonResident)
 
-foreign import CALLCONV unsafe "glAreTexturesResident"
-   glAreTexturesResident :: GLsizei -> Ptr GLuint -> Ptr GLboolean -> IO GLboolean
-
 --------------------------------------------------------------------------------
 
 type TexturePriority = GLclampf
 
 texturePriority :: TextureTarget -> StateVar TexturePriority
-texturePriority = texParamf id id TexturePriority
+texturePriority = texParamf realToFrac realToFrac TexturePriority
 
 prioritizeTextures :: [(TextureObject,TexturePriority)] -> IO ()
 prioritizeTextures tps =
    withArrayLen (map (textureID . fst) tps) $ \len texObjsBuf ->
       withArray (map snd tps) $
          glPrioritizeTextures (fromIntegral len) texObjsBuf
-
-foreign import CALLCONV unsafe "glPrioritizeTextures"
-   glPrioritizeTextures :: GLsizei -> Ptr GLuint -> Ptr GLclampf -> IO ()
