@@ -3,7 +3,7 @@
 -- Module      :  Graphics.Rendering.OpenGL.GL.VertexArrays
 -- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
--- 
+--
 -- Maintainer  :  sven.panne@aedion.de
 -- Stability   :  stable
 -- Portability :  portable
@@ -18,6 +18,7 @@ module Graphics.Rendering.OpenGL.GL.VertexArrays (
    NumComponents, DataType(..), Stride, VertexArrayDescriptor(..),
 
    -- * Specifying Data for the Arrays
+   Capability(..),
    ClientArrayType(..), arrayPointer,
    InterleavedArrays(..), interleavedArrays,
 
@@ -28,58 +29,44 @@ module Graphics.Rendering.OpenGL.GL.VertexArrays (
    ArrayIndex, NumArrayIndices, NumIndexBlocks,
    arrayElement, drawArrays, multiDrawArrays, drawElements, multiDrawElements,
    drawRangeElements, maxElementsVertices, maxElementsIndices, lockArrays,
-   primitiveRestartIndex,
+   primitiveRestartIndex, primitiveRestartIndexNV,
 
    -- * Generic Vertex Attribute Arrays
-   vertexAttribPointer, vertexAttribArray
+   vertexAttribPointer, vertexAttribArray,
 ) where
 
-import Foreign.Marshal.Alloc ( alloca )
-import Foreign.Ptr ( Ptr, nullPtr )
-import Foreign.Storable ( Storable(peek) )
-import Graphics.Rendering.OpenGL.GL.Capability (
-   EnableCap(CapVertexArray,CapNormalArray,CapColorArray,CapIndexArray,
-             CapTextureCoordArray,CapEdgeFlagArray,CapFogCoordArray,
-             CapSecondaryColorArray,CapMatrixIndexArray,CapPrimitiveRestart),
-   makeCapability, unmarshalCapability )
-import Graphics.Rendering.OpenGL.GL.BasicTypes (
-   GLboolean, GLenum, GLint, GLuint, GLsizei, Capability(Disabled,Enabled) )
-import Graphics.Rendering.OpenGL.GL.DataType (
-   DataType(..), marshalDataType, unmarshalDataType )
-import Graphics.Rendering.OpenGL.GL.Extensions (
-   FunPtr, unsafePerformIO, Invoker, getProcAddress )
-import Graphics.Rendering.OpenGL.GL.GLboolean (
-   marshalGLboolean, unmarshalGLboolean )
-import Graphics.Rendering.OpenGL.GL.QueryUtils (
-   GetPName(GetVertexArraySize,GetVertexArrayType,GetVertexArrayStride,
-            GetNormalArrayType,GetNormalArrayStride,GetColorArraySize,
-            GetColorArrayType,GetColorArrayStride,GetSecondaryColorArraySize,
-            GetSecondaryColorArrayType,GetSecondaryColorArrayStride,
-            GetIndexArrayType,GetIndexArrayStride,
-            GetFogCoordArrayType,GetFogCoordArrayStride,
-            GetTextureCoordArraySize,GetTextureCoordArrayType,
-            GetTextureCoordArrayStride,GetEdgeFlagArrayStride,
-            GetMaxElementsVertices,GetMaxElementsIndices,
-            GetClientActiveTexture,GetArrayElementLockFirst,
-            GetArrayElementLockCount,GetPrimitiveRestartIndex),
-   getInteger1, getEnum1, getSizei1, AttribLocation,
-   GetVertexAttribPName(..),
-   getVertexAttribInteger1, getVertexAttribEnum1, getVertexAttribBoolean1,
-   GetVertexAttribPointerPName(..), getVertexAttribPointer )
-import Graphics.Rendering.OpenGL.GL.PrimitiveMode ( marshalPrimitiveMode )
-import Graphics.Rendering.OpenGL.GL.BeginEnd ( PrimitiveMode )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   HasGetter(get),
-   GettableStateVar, makeGettableStateVar, StateVar, makeStateVar )
-import Graphics.Rendering.OpenGL.GL.Texturing.TextureUnit (
-   TextureUnit, marshalTextureUnit, unmarshalTextureUnit )
-import Graphics.Rendering.OpenGL.GLU.ErrorsInternal (
-   recordInvalidEnum, recordInvalidValue )
-import Graphics.Rendering.OpenGL.GL.VertexSpec ( IntegerHandling(..) )
-
---------------------------------------------------------------------------------
-
-#include "HsOpenGLExt.h"
+import Data.StateVar
+import Foreign.Marshal.Alloc
+import Foreign.Ptr
+import Foreign.Storable
+import Graphics.Rendering.OpenGL.GL.Capability
+import Graphics.Rendering.OpenGL.GL.DataType
+import Graphics.Rendering.OpenGL.GL.GLboolean
+import Graphics.Rendering.OpenGL.GL.PrimitiveMode
+import Graphics.Rendering.OpenGL.GL.QueryUtils
+import Graphics.Rendering.OpenGL.GL.Texturing.TextureUnit
+import Graphics.Rendering.OpenGL.GL.VertexSpec
+import Graphics.Rendering.OpenGL.GLU.ErrorsInternal
+import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility (
+   glArrayElement, glClientActiveTexture, glColorPointer, glDisableClientState,
+   glEdgeFlagPointer, glEnableClientState, glFogCoordPointer, glIndexPointer,
+   glInterleavedArrays, glNormalPointer, glSecondaryColorPointer,
+   glTexCoordPointer, glVertexPointer, gl_C3F_V3F, gl_C4F_N3F_V3F, gl_C4UB_V2F,
+   gl_C4UB_V3F, gl_COLOR_ARRAY, gl_COLOR_ARRAY_POINTER, gl_EDGE_FLAG_ARRAY,
+   gl_EDGE_FLAG_ARRAY_POINTER, gl_FEEDBACK_BUFFER_POINTER, gl_FOG_COORD_ARRAY,
+   gl_FOG_COORD_ARRAY_POINTER, gl_INDEX_ARRAY, gl_INDEX_ARRAY_POINTER,
+   gl_N3F_V3F, gl_NORMAL_ARRAY, gl_NORMAL_ARRAY_POINTER,
+   gl_SECONDARY_COLOR_ARRAY, gl_SECONDARY_COLOR_ARRAY_POINTER,
+   gl_SELECTION_BUFFER_POINTER, gl_T2F_C3F_V3F, gl_T2F_C4F_N3F_V3F,
+   gl_T2F_C4UB_V3F, gl_T2F_N3F_V3F, gl_T2F_V3F, gl_T4F_C4F_N3F_V4F, gl_T4F_V4F,
+   gl_TEXTURE_COORD_ARRAY, gl_TEXTURE_COORD_ARRAY_POINTER, gl_V2F, gl_V3F,
+   gl_VERTEX_ARRAY, gl_VERTEX_ARRAY_POINTER )
+import Graphics.Rendering.OpenGL.Raw.ARB.MatrixPalette (
+   gl_MATRIX_INDEX_ARRAY, gl_MATRIX_INDEX_ARRAY_POINTER )
+import Graphics.Rendering.OpenGL.Raw.ARB.VertexBlend ( gl_WEIGHT_ARRAY_POINTER )
+import Graphics.Rendering.OpenGL.Raw.Core31
+import Graphics.Rendering.OpenGL.Raw.EXT.CompiledVertexArray
+import Graphics.Rendering.OpenGL.Raw.NV.PrimitiveRestart
 
 --------------------------------------------------------------------------------
 
@@ -89,14 +76,7 @@ type Stride = GLsizei
 
 data VertexArrayDescriptor a =
    VertexArrayDescriptor !NumComponents !DataType !Stride !(Ptr a)
-#ifdef __HADDOCK__
--- Help Haddock a bit, because it doesn't do any instance inference.
-instance Eq (VertexArrayDescriptor a)
-instance Ord (VertexArrayDescriptor a)
-instance Show (VertexArrayDescriptor a)
-#else
    deriving ( Eq, Ord, Show )
-#endif
 
 noVertexArrayDescriptor :: VertexArrayDescriptor a
 noVertexArrayDescriptor = VertexArrayDescriptor 0 Byte 0 nullPtr
@@ -117,15 +97,15 @@ data ClientArrayType =
 
 marshalClientArrayType :: ClientArrayType -> GLenum
 marshalClientArrayType x = case x of
-   VertexArray -> 0x8074
-   NormalArray -> 0x8075
-   ColorArray -> 0x8076
-   IndexArray -> 0x8077
-   TextureCoordArray -> 0x8078
-   EdgeFlagArray -> 0x8079
-   FogCoordArray -> 0x8457
-   SecondaryColorArray -> 0x845e
-   MatrixIndexArray -> 0x8844
+   VertexArray -> gl_VERTEX_ARRAY
+   NormalArray -> gl_NORMAL_ARRAY
+   ColorArray -> gl_COLOR_ARRAY
+   IndexArray -> gl_INDEX_ARRAY
+   TextureCoordArray -> gl_TEXTURE_COORD_ARRAY
+   EdgeFlagArray -> gl_EDGE_FLAG_ARRAY
+   FogCoordArray -> gl_FOG_COORD_ARRAY
+   SecondaryColorArray -> gl_SECONDARY_COLOR_ARRAY
+   MatrixIndexArray -> gl_MATRIX_INDEX_ARRAY
 
 -- Hmmm...
 clientArrayTypeToEnableCap :: ClientArrayType -> EnableCap
@@ -177,9 +157,6 @@ setVertexPointer :: VertexArrayDescriptor a -> IO ()
 setVertexPointer (VertexArrayDescriptor n d s p) =
    glVertexPointer n (marshalDataType d) s p
 
-foreign import CALLCONV unsafe "glVertexPointer" glVertexPointer ::
-   GLint -> GLenum -> GLsizei -> Ptr a -> IO ()
-
 --------------------------------------------------------------------------------
 
 normalPointer :: StateVar (VertexArrayDescriptor a)
@@ -196,9 +173,6 @@ setNormalPointer :: VertexArrayDescriptor a -> IO ()
 setNormalPointer (VertexArrayDescriptor n d s p) =
    check (n == 3) $ glNormalPointer (marshalDataType d) s p
 
-foreign import CALLCONV unsafe "glNormalPointer" glNormalPointer ::
-   GLenum -> GLsizei -> Ptr a -> IO ()
-
 --------------------------------------------------------------------------------
 
 colorPointer :: StateVar (VertexArrayDescriptor a)
@@ -216,9 +190,6 @@ setColorPointer :: VertexArrayDescriptor a -> IO ()
 setColorPointer (VertexArrayDescriptor n d s p) =
    check (n == 3 || n == 4) $ glColorPointer n (marshalDataType d) s p
 
-foreign import CALLCONV unsafe "glColorPointer" glColorPointer ::
-   GLint -> GLenum -> GLsizei -> Ptr a -> IO ()
-
 --------------------------------------------------------------------------------
 
 indexPointer :: StateVar (VertexArrayDescriptor a)
@@ -235,9 +206,6 @@ setIndexPointer :: VertexArrayDescriptor a -> IO ()
 setIndexPointer (VertexArrayDescriptor n d s p) =
    check (n == 1) $ glIndexPointer (marshalDataType d) s p
 
-foreign import CALLCONV unsafe "glIndexPointer" glIndexPointer ::
-   GLenum -> GLsizei -> Ptr a -> IO ()
-
 --------------------------------------------------------------------------------
 
 texCoordPointer :: StateVar (VertexArrayDescriptor a)
@@ -255,9 +223,6 @@ setTexCoordPointer :: VertexArrayDescriptor a -> IO ()
 setTexCoordPointer (VertexArrayDescriptor n d s p) =
    glTexCoordPointer n (marshalDataType d) s p
 
-foreign import CALLCONV unsafe "glTexCoordPointer" glTexCoordPointer ::
-   GLint -> GLenum -> GLsizei -> Ptr a -> IO ()
-
 --------------------------------------------------------------------------------
 
 edgeFlagPointer :: StateVar (VertexArrayDescriptor a)
@@ -273,9 +238,6 @@ setEdgeFlagPointer :: VertexArrayDescriptor a -> IO ()
 setEdgeFlagPointer (VertexArrayDescriptor n d s p) =
    check (n == 1 && d == UnsignedByte) $ glEdgeFlagPointer s p
 
-foreign import CALLCONV unsafe "glEdgeFlagPointer" glEdgeFlagPointer ::
-   GLsizei -> Ptr a -> IO ()
-
 --------------------------------------------------------------------------------
 
 fogCoordPointer :: StateVar (VertexArrayDescriptor a)
@@ -290,9 +252,7 @@ getFogCoordPointer = do
 
 setFogCoordPointer :: VertexArrayDescriptor a -> IO ()
 setFogCoordPointer (VertexArrayDescriptor n d s p) =
-   check (n == 1) $ glFogCoordPointerEXT (marshalDataType d) s p
-
-EXTENSION_ENTRY("GL_EXT_fog_coord or OpenGL 1.4",glFogCoordPointerEXT,GLenum -> GLsizei -> Ptr a -> IO ())
+   check (n == 1) $ glFogCoordPointer (marshalDataType d) s p
 
 --------------------------------------------------------------------------------
 
@@ -310,9 +270,7 @@ getSecondaryColorPointer = do
 
 setSecondaryColorPointer :: (VertexArrayDescriptor a) -> IO ()
 setSecondaryColorPointer (VertexArrayDescriptor n d s p) =
-   glSecondaryColorPointerEXT n (marshalDataType d) s p
-
-EXTENSION_ENTRY("GL_EXT_secondary_color or OpenGL 1.4",glSecondaryColorPointerEXT,GLint -> GLenum -> GLsizei -> Ptr a -> IO ())
+   glSecondaryColorPointer n (marshalDataType d) s p
 
 --------------------------------------------------------------------------------
 
@@ -335,29 +293,26 @@ data InterleavedArrays =
 
 marshalInterleavedArrays :: InterleavedArrays -> GLenum
 marshalInterleavedArrays x = case x of
-   V2f -> 0x2a20
-   V3f -> 0x2a21
-   C4ubV2f -> 0x2a22
-   C4ubV3f -> 0x2a23
-   C3fV3f -> 0x2a24
-   N3fV3f -> 0x2a25
-   C4fN3fV3f -> 0x2a26
-   T2fV3f -> 0x2a27
-   T4fV4f -> 0x2a28
-   T2fC4ubV3f -> 0x2a29
-   T2fC3fV3f -> 0x2a2a
-   T2fN3fV3f -> 0x2a2b
-   T2fC4fN3fV3f -> 0x2a2c
-   T4fC4fN3fV4f -> 0x2a2d
+   V2f -> gl_V2F
+   V3f -> gl_V3F
+   C4ubV2f -> gl_C4UB_V2F
+   C4ubV3f -> gl_C4UB_V3F
+   C3fV3f -> gl_C3F_V3F
+   N3fV3f -> gl_N3F_V3F
+   C4fN3fV3f -> gl_C4F_N3F_V3F
+   T2fV3f -> gl_T2F_V3F
+   T4fV4f -> gl_T4F_V4F
+   T2fC4ubV3f -> gl_T2F_C4UB_V3F
+   T2fC3fV3f -> gl_T2F_C3F_V3F
+   T2fN3fV3f -> gl_T2F_N3F_V3F
+   T2fC4fN3fV3f -> gl_T2F_C4F_N3F_V3F
+   T4fC4fN3fV4f -> gl_T4F_C4F_N3F_V4F
 
 --------------------------------------------------------------------------------
 
 interleavedArrays :: InterleavedArrays -> Stride -> Ptr a -> IO ()
 interleavedArrays = glInterleavedArrays . marshalInterleavedArrays
 
-foreign import CALLCONV unsafe "glInterleavedArrays" glInterleavedArrays ::
-   GLenum -> GLsizei -> Ptr a -> IO ()
-
 --------------------------------------------------------------------------------
 
 clientState :: ClientArrayType -> StateVar Capability
@@ -372,20 +327,12 @@ setClientState arrayType val =
    (if val == Enabled then glEnableClientState else glDisableClientState)
       (marshalClientArrayType arrayType)
 
-foreign import CALLCONV unsafe "glEnableClientState" glEnableClientState ::
-   GLenum -> IO ()
-
-foreign import CALLCONV unsafe "glDisableClientState" glDisableClientState ::
-   GLenum -> IO ()
-
 --------------------------------------------------------------------------------
 
 clientActiveTexture :: StateVar TextureUnit
 clientActiveTexture =
    makeStateVar (getEnum1 unmarshalTextureUnit GetClientActiveTexture)
-                (glClientActiveTextureARB . marshalTextureUnit)
-
-EXTENSION_ENTRY("GL_ARB_multitexture or OpenGL 1.3",glClientActiveTextureARB,GLenum -> IO ())
+                (glClientActiveTexture . marshalTextureUnit)
 
 --------------------------------------------------------------------------------
 
@@ -400,43 +347,29 @@ type NumIndexBlocks = GLsizei
 arrayElement :: ArrayIndex -> IO ()
 arrayElement = glArrayElement
 
-foreign import CALLCONV unsafe "glArrayElement" glArrayElement :: GLint -> IO ()
-
 drawArrays :: PrimitiveMode -> ArrayIndex -> NumArrayIndices -> IO ()
 drawArrays = glDrawArrays . marshalPrimitiveMode
 
-foreign import CALLCONV unsafe "glDrawArrays" glDrawArrays ::
-   GLenum -> GLint -> GLsizei -> IO ()
-
 multiDrawArrays ::
       PrimitiveMode -> Ptr ArrayIndex -> Ptr NumArrayIndices -> NumIndexBlocks
    -> IO ()
-multiDrawArrays = glMultiDrawArraysEXT . marshalPrimitiveMode
-
-EXTENSION_ENTRY("GL_EXT_multi_draw_arrays or OpenGL 1.4",glMultiDrawArraysEXT,GLenum -> Ptr ArrayIndex -> Ptr GLsizei -> GLsizei -> IO ())
+multiDrawArrays = glMultiDrawArrays . marshalPrimitiveMode
 
 drawElements :: PrimitiveMode -> NumArrayIndices -> DataType -> Ptr a -> IO ()
 drawElements m c = glDrawElements (marshalPrimitiveMode m) c . marshalDataType
 
-foreign import CALLCONV unsafe "glDrawElements" glDrawElements ::
-   GLenum -> GLsizei -> GLenum -> Ptr a -> IO ()
-
 multiDrawElements ::
       PrimitiveMode -> Ptr NumArrayIndices -> DataType -> Ptr (Ptr a)
    -> NumIndexBlocks -> IO ()
 multiDrawElements m c =
-   glMultiDrawElementsEXT (marshalPrimitiveMode m) c . marshalDataType
-
-EXTENSION_ENTRY("GL_EXT_multi_draw_arrays or OpenGL 1.4",glMultiDrawElementsEXT,GLenum -> Ptr GLsizei -> GLenum -> Ptr (Ptr a) -> GLsizei -> IO ())
+   glMultiDrawElements (marshalPrimitiveMode m) c . marshalDataType
 
 drawRangeElements ::
       PrimitiveMode -> (ArrayIndex, ArrayIndex) -> NumArrayIndices -> DataType
    -> Ptr a -> IO ()
 drawRangeElements m (s, e) c =
-   glDrawRangeElementsEXT (marshalPrimitiveMode m) (fromIntegral s)
-                          (fromIntegral e) c . marshalDataType
-
-EXTENSION_ENTRY("GL_EXT_draw_range_elements or OpenGL 1.2",glDrawRangeElementsEXT,GLenum -> GLuint -> GLuint -> GLsizei -> GLenum -> Ptr a -> IO ())
+   glDrawRangeElements (marshalPrimitiveMode m) (fromIntegral s)
+                       (fromIntegral e) c . marshalDataType
 
 maxElementsVertices :: GettableStateVar NumArrayIndices
 maxElementsVertices = makeGettableStateVar (getSizei1 id GetMaxElementsVertices)
@@ -458,34 +391,37 @@ getLockArrays = do
       else return Nothing
 
 setLockArrays :: Maybe (ArrayIndex, NumArrayIndices) -> IO ()
-setLockArrays = maybe glUnlockArraysEXT (uncurry glLockArraysEXT)
+setLockArrays = maybe glUnlockArrays (uncurry glLockArrays)
 
-EXTENSION_ENTRY("GL_EXT_compiled_vertex_array",glLockArraysEXT,GLint -> GLsizei -> IO ())
-EXTENSION_ENTRY("GL_EXT_compiled_vertex_array",glUnlockArraysEXT,IO ())
+--------------------------------------------------------------------------------
+
+primitiveRestartIndex :: StateVar (Maybe ArrayIndex)
+primitiveRestartIndex =
+   makeStateVarMaybe
+      (return CapPrimitiveRestart)
+      (getInteger1 id GetPrimitiveRestartIndex)
+      (glPrimitiveRestartIndex . fromIntegral)
 
 --------------------------------------------------------------------------------
 
 -- We almost could use makeStateVarMaybe below, but, alas, this is client state.
 
-primitiveRestartIndex :: StateVar (Maybe ArrayIndex)
-primitiveRestartIndex =
-   makeStateVar getPrimitiverestartIndex setPrimitiverestartIndex
+primitiveRestartIndexNV :: StateVar (Maybe ArrayIndex)
+primitiveRestartIndexNV =
+   makeStateVar getPrimitiveRestartIndexNV setPrimitiveRestartIndexNV
 
-getPrimitiverestartIndex :: IO (Maybe ArrayIndex)
-getPrimitiverestartIndex = do
-   state <- get (makeCapability CapPrimitiveRestart)
-   if state == Enabled
-      then fmap Just $ getInteger1 fromIntegral GetPrimitiveRestartIndex
+getPrimitiveRestartIndexNV :: IO (Maybe ArrayIndex)
+getPrimitiveRestartIndexNV = do
+   on <- getBoolean1 unmarshalGLboolean GetPrimitiveRestartNV
+   if on
+      then fmap Just $ getInteger1 fromIntegral GetPrimitiveRestartIndexNV
       else return Nothing
 
-setPrimitiverestartIndex :: Maybe ArrayIndex -> IO ()
-setPrimitiverestartIndex maybeIdx = case maybeIdx of
-   Nothing  -> glDisableClientState primitiveRestartNV
-   Just idx -> do glEnableClientState primitiveRestartNV
+setPrimitiveRestartIndexNV :: Maybe ArrayIndex -> IO ()
+setPrimitiveRestartIndexNV maybeIdx = case maybeIdx of
+   Nothing  -> glDisableClientState gl_PRIMITIVE_RESTART_NV
+   Just idx -> do glEnableClientState gl_PRIMITIVE_RESTART_NV
                   glPrimitiveRestartIndexNV (fromIntegral idx)
-   where primitiveRestartNV = 0x8558   -- ToDo: HACK!
-
-EXTENSION_ENTRY("GL_NV_primitive_restart",glPrimitiveRestartIndexNV,GLuint -> IO ())
 
 --------------------------------------------------------------------------------
 
@@ -505,18 +441,18 @@ data GetPointervPName =
 
 marshalGetPointervPName :: GetPointervPName -> GLenum
 marshalGetPointervPName x = case x of
-   VertexArrayPointer -> 0x808e
-   NormalArrayPointer -> 0x808f
-   ColorArrayPointer -> 0x8090
-   IndexArrayPointer -> 0x8091
-   TextureCoordArrayPointer -> 0x8092
-   EdgeFlagArrayPointer -> 0x8093
-   FogCoordArrayPointer -> 0x8456
-   SecondaryColorArrayPointer -> 0x845d
-   FeedbackBufferPointer -> 0xdf0
-   SelectionBufferPointer -> 0xdf3
-   WeightArrayPointer -> 0x86ac
-   MatrixIndexArrayPointer -> 0x8849
+   VertexArrayPointer -> gl_VERTEX_ARRAY_POINTER
+   NormalArrayPointer -> gl_NORMAL_ARRAY_POINTER
+   ColorArrayPointer -> gl_COLOR_ARRAY_POINTER
+   IndexArrayPointer -> gl_INDEX_ARRAY_POINTER
+   TextureCoordArrayPointer -> gl_TEXTURE_COORD_ARRAY_POINTER
+   EdgeFlagArrayPointer -> gl_EDGE_FLAG_ARRAY_POINTER
+   FogCoordArrayPointer -> gl_FOG_COORD_ARRAY_POINTER
+   SecondaryColorArrayPointer -> gl_SECONDARY_COLOR_ARRAY_POINTER
+   FeedbackBufferPointer -> gl_FEEDBACK_BUFFER_POINTER
+   SelectionBufferPointer -> gl_SELECTION_BUFFER_POINTER
+   WeightArrayPointer -> gl_WEIGHT_ARRAY_POINTER
+   MatrixIndexArrayPointer -> gl_MATRIX_INDEX_ARRAY_POINTER
 
 --------------------------------------------------------------------------------
 
@@ -525,13 +461,10 @@ getPointer n = alloca $ \buf -> do
    glGetPointerv (marshalGetPointervPName n) buf
    peek buf
 
-foreign import CALLCONV unsafe "glGetPointerv" glGetPointerv ::
-   GLenum -> Ptr (Ptr a) -> IO ()
-
 --------------------------------------------------------------------------------
 
 vertexAttribPointer :: AttribLocation -> StateVar (IntegerHandling, VertexArrayDescriptor a)
-vertexAttribPointer location = undefined
+vertexAttribPointer location =
    makeStateVar (getVertexAttribPointer_ location) (setVertexAttribPointer location)
 
 getVertexAttribPointer_ :: AttribLocation -> IO (IntegerHandling, VertexArrayDescriptor a)
@@ -548,15 +481,12 @@ getVertexAttribPointer_ location = do
    return (h, VertexArrayDescriptor n d s p)
 
 setVertexAttribPointer :: AttribLocation -> (IntegerHandling, VertexArrayDescriptor a) -> IO ()
-setVertexAttribPointer location (h, VertexArrayDescriptor n d s p) = case h of
-   ToFloat -> glVertexAttribPointerARB location n md (marshalGLboolean False) s p
-   ToNormalizedFloat -> glVertexAttribPointerARB location n md (marshalGLboolean True) s p
+setVertexAttribPointer (AttribLocation location) (h, VertexArrayDescriptor n d s p) = case h of
+   ToFloat -> glVertexAttribPointer location n md (marshalGLboolean False) s p
+   ToNormalizedFloat -> glVertexAttribPointer location n md (marshalGLboolean True) s p
    KeepIntegral -> glVertexAttribIPointer location n md s p
   where md = marshalDataType d
 
-EXTENSION_ENTRY("GL_ARB_vertex_shader or OpenGL 2.0",glVertexAttribPointerARB,AttribLocation -> GLint -> GLenum -> GLboolean -> GLsizei -> Ptr a -> IO ())
-EXTENSION_ENTRY("GL_EXT_gpu_shader4 or OpenGL 3.0",glVertexAttribIPointer,AttribLocation -> GLint -> GLenum -> GLsizei -> Ptr a -> IO ())
-
 --------------------------------------------------------------------------------
 
 vertexAttribArray :: AttribLocation -> StateVar Capability
@@ -568,8 +498,5 @@ getVertexAttribArray location =
    getVertexAttribBoolean1 unmarshalCapability location GetVertexAttribArrayEnabled
 
 setVertexAttribArray :: Capability -> AttribLocation -> IO ()
-setVertexAttribArray Disabled = glDisableVertexAttribArrayARB
-setVertexAttribArray Enabled = glEnableVertexAttribArrayARB
-
-EXTENSION_ENTRY("GL_ARB_vertex_shader or OpenGL 2.0",glDisableVertexAttribArrayARB,AttribLocation -> IO ())
-EXTENSION_ENTRY("GL_ARB_vertex_shader or OpenGL 2.0",glEnableVertexAttribArrayARB,AttribLocation -> IO ())
+setVertexAttribArray Disabled (AttribLocation location) = glDisableVertexAttribArray location
+setVertexAttribArray Enabled (AttribLocation location) = glEnableVertexAttribArray location
